
import subprocess
import tifffile
import io
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image
import os

class FramePuller:
    '''
    Read image frames from a video file.
    Uses ffmpeg's piping and Pillow to read.
    '''
    def __init__(self, fn):
        self.fn = fn
    
    def _get_cmd(self, vframes):
        return ["ffmpeg", "-hide_banner", "-loglevel", "error", "-i", self.fn, "-r", "1/1", "-c:v", "tiff", "-vframes", str(vframes), '-pixel_format', 'gray16le', "-f", "image2pipe", "-"]
    
    def get_frames(self):
        process = subprocess.Popen(self._get_cmd(20), stdout=subprocess.PIPE)
        output = process.communicate()[0]
 
        barr = bytearray(output)

        b1 = barr[0:4]
        parts = barr.split(b1)
        
        images = []
        
        for part in parts[1:]:
            part = bytes(b1 + part)
            
            f = io.BytesIO()
            f.write(part)
            
            images.append(Image.open(f))
        
        return np.asarray(images)
        
    
def main():
    
    fn = TESTVIDEOFILE
    
    images = FramePuller(fn).get_frames()
    
    print(images.shape)
    
    i = 4000
    for image in images:
        tifffile.imsave(os.path.join(os.path.dirname(fn), os.path.basename(fn)+str(i)+'.tiff'), np.array(image))
        i+=1
        
if __name__ == "__main__":
    main()



'''
A script to make a video where DrosoM format saved fly
is spinned over the horizontal rotation (binocular overlap control).
'''

import sys
import os


from PIL import Image
import numpy as np
import tifffile
import tkinter as tk

from tk_steroids.matplotlib import SequenceImshow

def main():
    '''
    usage
    python3 drosox_spin.py dir1 dir2 dir3 ...
    '''

    order = ['nostop', 'ownstop', 'extstop']

    #ycrop = 300
    #yshift = -50
    ycrop, yshift = (1,0
    data = []
    clip = 95

    folders = []
    for name in order:
        for arg in sys.argv[1:]:
            if name in arg:
                folders.append(arg)
                break

    print(folders)

    # get data
    for arg in folders:
        print(arg) 
        folders = [f for f in os.listdir(arg) if os.path.isdir(os.path.join(arg, f)) and 'snap' not in f]
        hors = [int( f.split('(')[1].split(',')[0] ) for f in folders]

        hors, folders = zip(*sorted(zip(hors, folders), key=lambda x: x[0]))


        imfns = [[os.path.join(arg, f, fn) for fn in os.listdir(os.path.join(arg, f)) if fn.endswith('.tiff')][0] for f in folders]
        
        data.append([np.array(hors), imfns])


    # min max
    hor_start = max([hors[0] for hors, imfns in data])
    hor_stop = min([hors[-1] for hors, imfns in data])

    print('hor_start {}, hor_stop {}'.format(hor_start, hor_stop))

    

    # select center points
    for hors, imfns in data:
        break 
        loaded = [tifffile.imread(fn) for i_fn, fn in enumerate(imfns) if not print('{}/{}'.format(i_fn, len(imfns)))]
        print(len(loaded))
        tkroot = tk.Tk()
        imshow = SequenceImshow(tkroot)
        imshow.imshow(loaded, cmap='gray', slider=True)
        imshow.grid(row=1, column=1, sticky='NSWE')
        tkroot.columnconfigure(1, weight=1)
        tkroot.rowconfigure(1, weight=1)
        tkroot.mainloop()
    
    # normalization values
    normalization = []
    for hors, imfns in data:
        image = tifffile.imread(imfns[0])[ycrop+yshift:-ycrop+yshift:,]
        normalization.append( np.percentile(image, clip) )

    print(normalization)

    remove_fns = []

    for i_image, hor in enumerate(range(hor_start, hor_stop+1, 1)):
        images = []
        for i_data, (hors, imfns) in enumerate(data):
            i_closest = np.argmin(np.abs(hors-hor))
            
            image = np.clip(tifffile.imread(imfns[i_closest])[ycrop+yshift:-ycrop+yshift:,], 0, normalization[i_data])
            image = image - np.min(image)
            image = 255*image.astype(np.float) / np.max(image)
            images.append(image)
             
        image = Image.fromarray( np.concatenate((*images,),axis=0).astype(np.uint8) )
        
        #imfn = 'images/im{0:03d}.jpg'.format(i_image)
        imfn = 'images/im{0:03d}_horXZ.jpg'.format(i_image).replace('XZ', '{}').format(hor)
        image.save(imfn)

        remove_fns.append(imfn)


    # encode mp4
    os.chdir('images')
    command = 'ffmpeg -framerate 20 -i im%03d.jpg -c:v libx264 -vf fps=20 -pix_fmt yuv420p -preset slow -crf 28 -vf "pad=ceil(iw/2)*2:ceil(ih/2)*2" {}.mp4'.format('out')

    #os.system(command)
    os.chdir('..')
    
    #for fn in remove_fns:
    #    os.remove(fn)



if __name__ == "__main__":
    main()




'''Export data from MAnalyser objects
'''

import math
import datetime
import json

import numpy as np

from gonioanalysis.coordinates import get_rotation_matrix
from .optic_flow import field_error

# Available export filetypes
FILETYPES = [('JSON', '.json'), ('Numpy binary', '.npy')]


def vectors_to_yxz_rotations(points, vectors):
    '''Converts 3D vectors to yxz rotations for external applications.

    Excexution: Have an arrow 3D model at origo, pointing towards positive X.
    Then, rotate the arrow along Y axis (the arrow end still remaining at origo).
    Then, move the arrow to (0,1,0) and while having the center of rotation
    at origo (0,0,0), rotate first along the X axis and then along the Z axis.
    After these steps you have reconstructed the vector map.

    Arguments
    ---------
    points : list
        Vector locations from get_3d_vectors
    vectors : list
        Vectors themselves from get_3d_vectors

    Returns
    -------
    yxz_rots : list
        List of rotations [(yr, xr, zr), ...]
    '''
    
    ## Transform points and points+vectors in reverse to the Y-axis
    zrots = [math.atan2(y, x)-math.pi/2 for x,y,z in points]

    rotated_points = [get_rotation_matrix('z', -rot) @ point for rot, point in zip(zrots, points)]
    rotated_vectors = [get_rotation_matrix('z', -rot) @ vector for rot, vector in zip(zrots, vectors)]

    # points should be all now in in yz plane
    #xrots = [math.asin(z) for x,y,z in rotated_points]
    xrots = [math.atan2(z, y) for x,y,z in rotated_points]
    rotated_points = [get_rotation_matrix('x', -rot) @ point for rot, point in zip(xrots, rotated_points)]
    rotated_vectors = [get_rotation_matrix('x', -rot) @ vector for rot, vector in zip(xrots, rotated_vectors)]

    # calculate rotation of the vector when point now on (0,1,0)
    yrots = [math.atan2(-x, z) - math.pi for x,y,z in rotated_vectors]
    
    return [[yr,xr,zr] for yr,xr,zr in zip(yrots, xrots, zrots)]


def _date():
    return str(datetime.datetime.now())


def _export_data(data, save_fn):
    if save_fn.endswith('.json'):
        with open(save_fn, 'w') as fp:
            json.dump(data, fp)
    else:
        raise ValueError(f'Unsupported filetype: {save_fn}')

def export_vectormap(analyser, save_fn=None):
    '''Exports 3D vectors

    save_fn : string
        Path to the new file
    '''

    if save_fn is None:
        saven_fn = f'vectormap_{analyser.name}_{_date()}.npy'

    if save_fn.endswith('.npy'):
        base = save_fn.removesuffix('.npy')
        for eye in ['left', 'right']:
            vectors =  analyser.get_3d_vectors(eye)
            np.save(base+'_'+eye+'.npy', vectors)
    
    elif save_fn.endswith('.json'):
        data = {}
        for eye in ['left', 'right']:
            points, vectors = analyser.get_3d_vectors(eye)
            data[eye] = {
                    'points': points.tolist(), 
                    'vectors': vectors.tolist(),
                    'yxz_rotations': vectors_to_yxz_rotations(points, vectors)}
        
        with open(save_fn, 'w') as fp:
            json.dump(data, fp)

    else:
        raise ValueError(f'Unkown file ending: {save_fn}')




def export_differencemap(analyser1, analyser2, save_fn=None):
    '''Exports the comparision between two analysers' vectormaps.

    The differences (aka. errors) range between 0 and 1 and are
    in the points of the analyser1.
    '''
    
    data = {}
    
    for eye in analyser1.eyes:
        p1, v1 = analyser1.get_3d_vectors(eye)
        p2, v2 = analyser2.get_3d_vectors(eye)
        errors = field_error(p1, v1, p2, v2)
        
        print(len(p1))
        print(len(p2))

        data[eye] = {
                'points1': p1.tolist(),
                'points2': p2.tolist(),
                'vectors1': v1.tolist(),
                'vectors2': v2.tolist(),
                'yxz_rotations1': vectors_to_yxz_rotations(p1, v1),
                'yxz_rotations2': vectors_to_yxz_rotations(p2, v2),
                'errors': errors.tolist()
                }

    if save_fn is None:
        save_fn = f'differencemap_{analyser1.name}_{analyser2.name}_{_date()}.json'

    _export_data(data, save_fn)




'''
Analysing and creating figures from the norpA rescues.
'''

import os
import csv
import glob

import matplotlib.pyplot as plt
import matplotlib.image as mati
import matplotlib.patches as patches
import matplotlib.colors as mplcolors
from matplotlib.backends.backend_pdf import PdfPages
import numpy as np
#import pandas as pd
#import seaborn as sns
#from statannot import add_stat_annotation

from gonioanalysis.directories import ANALYSES_SAVEDIR
from gonioanalysis.image_tools import open_adjusted


#from xray_erg.plotting import Animator
#from xray_erg.interpolating import interpolate
from biosystfiles import extract as bsextract
#from mpl_steroids.scalebar import add_scalebar


RESCUES = ['Rh1', 'Rh3', 'Rh4', 'Rh5', 'Rh6']
STIMULI = ['uv', 'green', 'nostim']


# FANCY NAMES
STIMULUS_FANCY_NAMES = {STIMULI[0]: 'UV flash', STIMULI[1]: 'Green flash', STIMULI[2]: 'No stimulus',
        STIMULI[0]+'_erg': 'UV flash', STIMULI[1]+'_erg': 'Green flash',
        'sens': ''}


# COLORS
PALETTE = ['violet', 'lime', 'gray']

RESCUE_COLORS = {'Rh1': '#4ce801', 'Rh3': '#e201f0', 'Rh4': '#7e01f1', 'Rh5': '#01d2d3', 'Rh6': '#c4d201'}

STIMULUS_COLORS = {'uv': 'purple', 'green': 'green', 'nostim': 'black'}
STIMULUS_COLORS['uv_erg'] = STIMULUS_COLORS['uv']
STIMULUS_COLORS['green_erg'] = STIMULUS_COLORS['green']


EXPERIMENT_COLORS = {'ergs': (0.99,0.99,0.97), 'dpp': (0.97,0.99,0.99)}


THIS_SAVEDIR = os.path.join(ANALYSES_SAVEDIR, 'norpa_rescues')


# BLockdict for blacklisting
BLOCKDICT = {'norpA_Rh1_02_manyrepeats': 'No ERGs',
        'norpA_Rh1_10_manyrepeats': 'Miniscule ERGs',
        'norpA_Rh4_06_manyrepeats': 'No ERGs',
        'norpA_Rh6_06_manyrepeats': 'ERGs not recorded',
        'norpA_Rh1_06_manyrepeats_right': 'Not responding to Green',
        'norpA_Rh3_01_manyrepeats_left': 'Not clearly responding to UV',
        'norpA_Rh3_03_manyrepeats_right': 'Not clearly responding to UV',
        'norpA_Rh3_04_manyrepeats_right': 'Not clearly responding to UV',
        'norpA_Rh3_07_manyrepeats_right': 'Not clearly responding to UV',
        'norpA_Rh5_03_manyrepeats_right': 'Not clearly responding to UV',
        'norpA_Rh5_05_manyrepeats_right': 'Not clearly responding to UV',
        'norpA_Rh5_09_manyrepeats_left': 'Not clearly responding to UV'
        }

# Temporal pixel size, should go to analysing.py
PXLSIZE=0.81741 # microns per pixel

# Led spectrums
LED_SPECTRUM_DIR = '/home/joni/data/DPP/DPP_cal_spectrometer_data/DPP_cal_10/'
LED_SPECTRUM_FNS = ['green_center_8V_calculated.csv', 'uv_center_8V.csv', 'ergsetup_green_center_5V.csv', 'ergsetup_uv_center_5V.csv'] 
LED_SPECTRUM_FNS = [os.path.join(LED_SPECTRUM_DIR, fn) for fn in LED_SPECTRUM_FNS]


def norpa_rescue_manyrepeats(manalysers):
    '''
    Analyse norpA Rh{1,3,4,5,6} rescue mutants recorded with
    experimental protocol manyrepeats.

    In the so called manyrepeats protocol, each eye was imaged
    with 25 repeats while flashing green, UV or no stimulus
    at location vertical= -37 deg and horizontal= +- 28 deg.
    
    The experiment was rotated almost without exception as follows:
        1) right eye UV
        2) right eye NOSTIM
        3) right eye GREEN
        4) left eye GREEN
        5) left eye NOSTIM
        5) left eye UV

    Specimens were named as norpA_Rh{i_rhodopsin}_{i_specimen}_manyrepeats,
    where i_rhodopsin is 1, 3, 4, 5, or 6 and i_specimen 01, 02, ...
    '''

    results = {}
    
    rescues = RESCUES
    stimuli = STIMULI

    
    

    for manalyser in manalysers:
        
        specimen_name = manalyser.get_specimen_name()
        
        if specimen_name in BLOCKDICT.keys():
            print('Specimen {} on the block list because {}'.format(specimen_name, BLOCKDICT[specimen_name]))
            continue

        # Look which mutant we have
        specimen_rescue = None
        for rescue in rescues:
            if rescue in specimen_name:
    
                if specimen_rescue is not None:
                    raise ValueError('2 or more Rhi rescues fit the name {}'.format(specimen_name))
                
                specimen_rescue = rescue
                break
        
        # If none of the rescues, then skip this Manalyser
        if specimen_rescue is None:
            continue


        # Then iterate over the image folder
        for image_folder in sorted(manalyser.list_imagefolders(list_special=False)):

            # Those image folders without suffix were in my recordings always
            # actually stimulated with UV; It was just in the beginning, I
            # sometimes forgot to add the suffix UV. Recordings were always
            # started with UV.
            specimen_stimtype = 'uv'

            for stimtype in stimuli:
                if image_folder.endswith('_'+stimtype):
                    specimen_stimtype = stimtype

            data = manalyser.get_movements_from_folder(image_folder)
            eye = list(data.keys())[0]
            data = data[eye]

            print('Specimen {}_{}'.format(specimen_name, eye))
            if '{}_{}'.format(specimen_name, eye) in BLOCKDICT.keys():
                print('  Specimen {}_{} blocked'.format(specimen_name, eye))
                continue

            times = []
            traces = []
            responses = []

            for i_repeat in range(len(data)):
                X = np.asarray(data[i_repeat]['x'])
                Y = np.asarray(data[i_repeat]['y'])

                mag_trace = np.sqrt(X**2+Y**2)
                
                A = np.mean(mag_trace[:4])
                B = np.mean(mag_trace[-10:])
                response = B-A
            
                traces.append(mag_trace)
                responses.append(response)
                times.append(manalyser.get_recording_time(image_folder, i_rep=i_repeat))


            if specimen_rescue not in results.keys():
                results[specimen_rescue] = {}

            if specimen_stimtype not in results[specimen_rescue].keys():
                results[specimen_rescue][specimen_stimtype] = {'times': [], 'separate_traces': [], 'traces': [],
                        'sexes': [], 'names': [], 'ages': [], 'responses': [], 'eyes': [], 'image_interval': []}

            results[specimen_rescue][specimen_stimtype]['separate_traces'].append(traces)
            results[specimen_rescue][specimen_stimtype]['traces'].append(np.mean(traces, axis=0))
            results[specimen_rescue][specimen_stimtype]['responses'].append(responses)

            results[specimen_rescue][specimen_stimtype]['times'].append( times[int(len(times)/2)] )
            results[specimen_rescue][specimen_stimtype]['ages'].append( manalyser.get_specimen_age() )
            results[specimen_rescue][specimen_stimtype]['sexes'].append( manalyser.get_specimen_sex() )
            results[specimen_rescue][specimen_stimtype]['names'].append( manalyser.get_specimen_name() )
            results[specimen_rescue][specimen_stimtype]['eyes'].append( eye )
            results[specimen_rescue][specimen_stimtype]['image_interval'].append( manalyser.get_image_interval() ) 
        

    #plot_individual(manalysers, results)
    #plot_mean_timecourses(results)
    #plot_mean_timecourses(results, sex='male')
    #plot_box_summary(results)    

    export_mean_timecourses(results)


def plot_bar_summary(results):
    

    panda_data = []
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            responses = results[rescue][stimtype]['responses']
            #for i_repeat, value in enumerate(values):
            panda_data.append([rescue, STIMULUS_FANCY_NAMES[stimtype], np.mean(responses), np.std(responses), responses])

    df = pd.DataFrame(panda_data, columns=['norpA rescue', 'Stimulus type', 'mean', 'std', 'responses'])
    print(df)
    

    a = df.pivot('norpA rescue', 'Stimulus type', 'mean').plot(kind='bar',
            yerr=df.pivot('norpA rescue', 'Stimulus type', 'std'))

def plot_box_summary(results):
    
    rescue_labels = {'No s'}

    panda_data = []
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            data = results[rescue][stimtype]
            for i_eye in range(len(data['eyes'])):
                panda_data.append([rescue, STIMULUS_FANCY_NAMES[stimtype],
                    data['names'][i_eye], data['eyes'][i_eye], np.mean(data['responses'][i_eye])])

    df = pd.DataFrame(panda_data, columns=['norpA rescue', 'Stimulus type', 'name', 'eye', 'response'])
    print(df)
    

    #a = df.pivot('norpA rescue', 'Stimulus type', 'mean').plot(kind='bar',
    #        yerr=df.pivot('norpA rescue', 'Stimulus type', 'std'))
    
    plt.figure()
    ax = sns.boxplot(x='norpA rescue', y='response', hue='Stimulus type', data=df,
            hue_order=[STIMULUS_FANCY_NAMES[stim] for stim in STIMULI], palette=PALETTE)
    
    box_pairs = []
    for rescue in results.keys():
        for i in range(len(STIMULI)-1):
            box_pairs.append(((rescue, STIMULUS_FANCY_NAMES[STIMULI[i]]), (rescue, STIMULUS_FANCY_NAMES[STIMULI[-1]])))
    
    print(box_pairs)

    add_stat_annotation(ax, data=df, x='norpA rescue', y='response', hue='Stimulus type',
            box_pairs=box_pairs, test='Wilcoxon', loc='inside')
    #ax = sns.swarmplot(x="norpA rescue", y="response", data=df, color=".25")

def plot_mean_timecourses(results, sex=None):
    '''
    Create a figure with time courses.
    '''


    fig = plt.figure(figsize=(16,9))
    subplots = {}
    
    nrows = 5
    ncols = 4

    cols = ['', 'sens', 'uv', 'green', 'nostim', 'uv_erg', 'green_erg']
    rows = ['colnames', 'Rh1', 'Rh3', 'Rh4', 'Rh5', 'Rh6']
    

    maxval = 12.5
   

    animator = Animator(fig)


    for row in rows[::-1]:
        for col in cols:
            irow = rows.index(row)
            icol = cols.index(col)
            
            ax = fig.add_subplot(len(rows),len(cols), 1+len(cols)*irow+icol)
            
            if row == rows[-1] and col == 'sens':
                ax.set_xlabel('Wavelength (nm)')
                ax.get_yaxis().set_visible(False)
                ax.spines['top'].set_visible(False)
                ax.spines['right'].set_visible(False)
                ax.spines['left'].set_visible(False)
            else:
                ax.set_axis_off()

            if col == 'sens' or row == 'colnames':
                ax.set_xlim(300,650)
                ax.set_ylim(0,1.05)

            subplots[row+col] = ax

            if icol == 1 and row!='colnames':
                color = RESCUE_COLORS.get(row, 'black')
                ax.text(0, 0.5, row, va='center', ha='right',
                        color=color, transform=ax.transAxes)
            
            if irow == 0:
                color = STIMULUS_COLORS.get(col, 'black')
                ax.text(1-0.618, 0.618, STIMULUS_FANCY_NAMES.get(col, col), va='bottom', ha='center',
                        color=color, transform=ax.transAxes)
    
    # Set titles ERG and DPP
    subplots['colnames'+'green'].text(0.5, 1.3, 'DPP microsaccades',
            ha='center', va='bottom', fontsize='large',
            backgroundcolor=EXPERIMENT_COLORS['dpp'], transform=subplots['colnames'+'green'].transAxes)

    subplots['colnames'+'green_erg'].text(0, 1.3, 'ERGs',
            ha='center', va='bottom', fontsize='large',
            backgroundcolor=EXPERIMENT_COLORS['ergs'], transform=subplots['colnames'+'green_erg'].transAxes)
    
    # Title lines
    #box1 = subplots['colnames'+'uv'].get_position()
    #box2 = subplots['colnames'+'nostim'].get_position()
    #arrow = patches.FancyArrow(box1.x0, box1.y1+box1.height/10, box2.x1-box1.x0, 0,
    #        shape='right', hatch='|', transform=fig.transFigure, figure=fig)
    #fig.patches.extend([arrow])

    
    # Plot norpA rescue illustrative images
    imagedir = '/home/joni/Pictures/NorpA rescues/'
    for fn in os.listdir(imagedir):
        for row in rows:
            if row in fn:
                image = mati.imread(os.path.join(imagedir, fn))
                h,w,d = image.shape
                
                im = subplots[row+''].imshow(image)
                
                im.set_clip_path( patches.Circle((int(w/2),int(h/2)),int(min(w,h)/2), transform=subplots[row].transData) )
                continue
    
    led_wavelengths, led_spectrums = _load_led_spectrums(LED_SPECTRUM_FNS)
    
    # Plot spectral sensitivities
    wave_axis, sensitivities = _load_spectral_sensitivities()
    for rescue, sensitivity in zip([row for row in rows if row in RESCUES], sensitivities):
        subplots[rescue+'sens'].plot(wave_axis, sensitivity, color=RESCUE_COLORS[rescue])
        
        # Plot LED spectrums in each figure
        subplots[rescue+'sens'].plot(led_wavelengths[0], led_spectrums[0], '--', color='green', lw=1)
        subplots[rescue+'sens'].plot(led_wavelengths[0], led_spectrums[1], '--', color='purple', lw=1)
    

    # Plot stimulus/LED spectral curves
    subplots['colnames'+'green'].plot(led_wavelengths[0], led_spectrums[0], color='green')
    subplots['colnames'+'uv'].plot(led_wavelengths[0], led_spectrums[1], color='purple')
    subplots['colnames'+'nostim'].plot([led_wavelengths[0][0], led_wavelengths[0][-1]], [0,0], color='black')
    subplots['colnames'+'green_erg'].plot(led_wavelengths[0], led_spectrums[2], color='green')
    subplots['colnames'+'uv_erg'].plot(led_wavelengths[0], led_spectrums[3], color='purple')
 
    # Plot ERGs

    erg_data = _load_ergs()
    
    ergs_min = np.inf
    ergs_max = -np.inf
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            
            erg_traces = []
            
            # For this rescue/stimtype combination, go through every erg
            for specimen_name in erg_data.keys():
                if specimen_name in BLOCKDICT.keys():
                    print('Blocked {}'.format(specimen_name))
                    continue
                # Looking for correct specimen
                if rescue in specimen_name:
                    # and for
                    for ergs in erg_data[specimen_name]:
                        # correct stimulus type
                        if stimtype in ergs and '25' in ergs:
                            erg = ergs[0][0]
                            erg = erg - erg[0]

                            color = np.array(mplcolors.to_rgb(RESCUE_COLORS[rescue]))
                            color = np.mean([color, (1,1,1)], axis=0)
                            subplots[rescue+stimtype+'_erg'].plot(erg, color=color, lw=1)
                            erg_traces.append(erg)
                            
                            ergs_min = min(ergs_min, np.min(erg))
                            ergs_max = max(ergs_max, np.max(erg))

            if erg_traces:
                color = np.array(mplcolors.to_rgb(RESCUE_COLORS[rescue])) * 0.75
                subplots[rescue+stimtype+'_erg'].plot(np.mean(erg_traces, axis=0), color=color)
    
    # Set ERG axis limits
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
                if rescue+stimtype+'_erg' in subplots.keys():
                    subplots[rescue+stimtype+'_erg'].set_ylim(ergs_min, ergs_max)

    
    # Set DPP and ERG plots background color

    box1 = subplots['Rh6'+'uv_erg'].get_position()
    box2 = subplots['Rh1'+'green_erg'].get_position()
    rect = patches.Rectangle((box1.x0, box1.y0), (box2.x0-box1.x0)+box2.width, (box2.y0-box1.y0)+box2.height, color=EXPERIMENT_COLORS['ergs'], zorder=-1)
    fig.add_artist(rect)         

    box1 = subplots['Rh6'+'uv'].get_position()
    box2 = subplots['Rh1'+'nostim'].get_position()
    rect = patches.Rectangle((box1.x0, box1.y0), (box2.x0-box1.x0)+box2.width, (box2.y0-box1.y0)+box2.height, color=EXPERIMENT_COLORS['dpp'], zorder=-1)
    fig.add_artist(rect)         
    
    
    # Add scale bars
    add_scalebar(subplots['Rh6'+'nostim'], 50, 5, position=(1,5), xunits='ms', yunits='µm')
    add_scalebar(subplots['Rh6'+'green_erg'], 300, -3, position=(100,-3), xunits='ms', yunits='mV')
    

    # Plot DPP data
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            
            ax = subplots[rescue+stimtype]

            traces = results[rescue][stimtype]['traces']
            
            # How many fps images were taken by camera
            image_interval = results[rescue][stimtype]['image_interval']

            color = None
            for i in range(len(traces)):
                if sex is not None:
                    print(results[rescue][stimtype]['sexes'][i])
                    if sex == results[rescue][stimtype]['sexes'][i]:
                        color = 'red'
                    else:
                        color = 'gray'

                x, y = interpolate(np.arange(0, len(traces[i])*image_interval[i]*1000, image_interval[i]*1000), traces[i]*PXLSIZE, len(traces[i]))
                color = np.array(mplcolors.to_rgb(RESCUE_COLORS[rescue]))
                color = np.mean([color, (1,1,1)], axis=0)               
                line = ax.plot(x, y, label=results[rescue][stimtype]['ages'][i], lw=1, color=color)
                
                animator.add_animation(line[0], [x,y], hide=False)

            color = np.array(mplcolors.to_rgb(RESCUE_COLORS[rescue])) * 0.75
            ax.plot(x, np.mean(traces,axis=0)*PXLSIZE, label='mean', lw=2, color=color)

            #ax.set_title(rescue+'_'+stimtype)
            ax.set_ylim(0,maxval)

    
    animator.frames += len(x)
    
    os.makedirs(THIS_SAVEDIR, exist_ok=True)
    
    animation = animator.get_animation(interval=40)
    #animation.save(os.path.join(THIS_SAVEDIR, 'timecourses.mp4'), dpi=600)

    plt.subplots_adjust(wspace=0.1, hspace=0.1)



def export_mean_timecourses(results):
    '''
    Exports the plot_mean_timecourses as csv files for plotting/manipulating
    with an external program.
    '''

    savedir = os.path.join(THIS_SAVEDIR, 'exports')
    os.makedirs(savedir, exist_ok=True)

    # Each item is for a file
    csv_files = {}
    
    # DPP data
    for rescue in results.keys():
        for stimtype in results[rescue].keys():

            # Each item for column
            csv_file = {}

            traces = results[rescue][stimtype]['traces']
            image_interval = results[rescue][stimtype]['image_interval']
            
            avg_traces = []

            for i in range(len(traces)):
                x, y = interpolate(np.arange(0, len(traces[i])*image_interval[i]*1000, image_interval[i]*1000), traces[i]*PXLSIZE, len(traces[i]))
                
                if i == 0:
                    csv_file['time (ms)'] = x
                
                specimen_name = results[rescue][stimtype]['names'][i]
                eye = results[rescue][stimtype]['eyes'][i]
                column_name = '{}_mean_{}'.format(specimen_name, eye)

                csv_file[column_name] = y

                avg_traces.append(y)
            
    
            if csv_file:
                # Add mean trace
                csv_file['mean'] = np.mean(avg_traces, axis=0)
                csv_files[rescue+'_'+stimtype] = csv_file

    # ERGs
    erg_data = _load_ergs()
    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            
            csv_file = {}

            traces = []
            
            # For this rescue/stimtype combination, go through every erg
            for specimen_name in erg_data.keys():
                if specimen_name in BLOCKDICT.keys():
                    print('Blocked {}'.format(specimen_name))
                    continue
                # Looking for correct specimen
                if rescue in specimen_name:
                    # and for
                    for ergs in erg_data[specimen_name]:
                        # correct stimulus type
                        if stimtype in ergs and '25' in ergs:
                            erg = ergs[0][0]
                            erg = erg - erg[0]
                            erg = erg.flatten()
                            
                            if not csv_file:
                                fs = ergs[0][1]
                                dt = 1/fs
                                csv_file['time (ms)'] = 1000 * np.arange(0, len(erg)*dt, dt)
                            
                            csv_file[specimen_name+'_erg'] = erg
                            traces.append(erg)
            if csv_file:
                csv_file['mean'] = np.mean(traces, axis=0)
                csv_files[rescue+'_'+stimtype +'_erg'] = csv_file

    # Measured specturms for LEDs
    led_wavelengths, led_spectrums = _load_led_spectrums(LED_SPECTRUM_FNS)
    
    for i_led, (wave, spec) in enumerate(zip(led_wavelengths, led_spectrums)):
        csv_file = {}
        csv_file['wavelength (nm)'] = wave
        csv_file['relative_intensity'] = spec
        csv_files[os.path.basename(LED_SPECTRUM_FNS[i_led]).rstrip('.csv')] = csv_file
        print('i_led {}'.format(i_led))
    
    # Spectral sensitivities
    wave_axis, sensitivities = _load_spectral_sensitivities()
    for rescue, sensitivity in zip(RESCUES, sensitivities):
        
        csv_file = {}
        csv_file['wavelength'] = wave_axis 
        csv_file['sensitivity'] = sensitivity

        csv_files['sensitivity_{}'.format(rescue)] = csv_file
    

    # Export
    for csv_name, csv_file in csv_files.items():
        
        with open(os.path.join(savedir, csv_name+'.csv'), 'w') as fp:
            
            writer = csv.writer(fp)

            column_names = sorted(list(csv_file.keys()))
            if 'time (ms)' in column_names:
                column_names.insert(0, column_names.pop(column_names.index('time (ms)')))
            
            if 'mean' in column_names:
                column_names.insert(len(column_names), column_names.pop(column_names.index('mean')))
            
            
            N = len(csv_file[column_names[0]])
            
            print("{} len {}".format(csv_name, len(column_names)-2))
            
            writer.writerow(column_names)

            for i in range(0, N):
                row = [csv_file[col][i] for col in column_names]
                
                writer.writerow(row)




def plot_ergs():
    '''
    Plot ERGs alone.
    '''

    ergs = get_ergs()
    
    for rescue in RESCUES:
        for stimtype in STIMULI:
            pass


    
def _load_ergs():
    '''
    Fetches ERGs for the specimen matching the name.

    Requirements
    - ERGs are Biosyst recorded .mat files.
    - Requires also a lab book that links each specimen name to a ERG file,
        and possible other parameter values such as intensity, repeats,
        UV/green etc.

    Returns ergs {specimen_name: data}
        where data is a list [[ergs, par1, par2, ...],[..],..]
        ergs are np arrays
    '''
    ergs = {}

    ergs_rootdir = '/home/joni/data/DPP_ERGs'
    ergs_labbook = '/home/joni/data/DPP_ERGs/labbook_norpa_rescues.csv'
    
    csvfile = []
    with open(ergs_labbook, 'r') as fp:
        reader = csv.reader(fp)
        for row in reader:
            csvfile.append(row)
    
    previous_specimen = ''

    for line in csvfile:
        efn = line[1]
        match = glob.glob(ergs_rootdir+'/**/'+efn)
        if len(match) != 1:
            print('{} not found'.format(efn))
            #ergs.append(None)
        else:
            #print(efn + ' ' + match[0])

            specimen = line[0]
            if not specimen:
                specimen = previous_specimen
            previous_specimen = specimen

            try:
                ergs[specimen]
            except KeyError:
                ergs[specimen] = []

            ergs[specimen].append([bsextract(match[0], 0), *line[2:]])

    return ergs


def _load_spectral_sensitivities(fn='/home/joni/analyses/digitized_rh_rescues.csv'):
    '''
    Spectral sensitivies of different Rh opsins. Digitalized from a figure into a csv file.

    Returns
        wavelengths, [rh1, ...]
        where wavelengths a numpy array 1D
        and rh1, rh3,... also
    '''

    data = np.loadtxt(fn, delimiter=' ', skiprows=1)
    print(data.shape) 
    return data[:,0], [data[:, i] for i in range(1, data.shape[1])]



def _load_led_spectrums(spectrometer_csv_files):
    '''
    Returns spectrums.
    
    spectometer_csv_files       A list of spectrometer csv files.

    Returns     wavelengts  spectrums
        which both are 1d lists containing 1d numpy arrays
    '''
    
    wavelengths = []
    spectrums = []
    
    # Load spectrums
    for fn in spectrometer_csv_files:
        spectrum = np.loadtxt(fn, delimiter=',', skiprows=1)[:,1]
        
        wavelength = np.loadtxt(fn, delimiter=',', skiprows=1)[:,0]
        wavelength = [207.1545+0.3796126*w+0.00002822671*(w**2) for w in wavelength]
        
        spectrums.append(spectrum)
        wavelengths.append(wavelength)

    
    # Load integration times from txt files
    for i, fn in enumerate([fn.rstrip('.csv')+'.txt' for fn in spectrometer_csv_files]):
        with open(fn, 'r') as fp:
            integration_time = fp.readline()
        integration_time = float(integration_time.split(' ')[-2]) # ms
        
        spectrums[i] = spectrums[i] / integration_time

    cmax = np.max(spectrums)

    return wavelengths, [spectrum/cmax for spectrum in spectrums]




def _get_results_keys(results, specimen_name):
    '''
    '''
    
    matches = []

    for rescue in results.keys():
        for stimtype in results[rescue].keys():
            try:
                index = results[rescue][stimtype]['names'].index(specimen_name)
                matches.append([rescue, stimtype, index])
            except:
                pass

    return matches


def plot_individual(manalysers, results):
    
    

    pdf_savedir = os.path.join(ANALYSES_SAVEDIR, 'norpa_rescues')
    os.makedirs(pdf_savedir, exist_ok=True)

    ergs = _load_ergs()
    
    
    with PdfPages(os.path.join(pdf_savedir, 'individual.pdf')) as pdf:

        for manalyser in manalysers[:5]:
            specimen_name = manalyser.get_specimen_name()
            try:
                ergs[specimen_name]
                print('ERGS')
            except KeyError:
                print('No ERGs for {}'.format(specimen_name))
            
            keys = _get_results_keys(results, specimen_name)
            print(keys)

            # Specimen info
            fig, axes = plt.subplots(1+len(keys),2, figsize=(8.3,11.7))
            plt.title(specimen_name)
            
            # Plot face image
            axes[0][0].imshow(open_adjusted(manalyser.get_snap()), cmap='gray')

            # Plot DPP

            for i, key in enumerate(keys):
                axes[i][0].plot( results[key[0]][key[1]]['separate_traces'][key[2]] )
            

            pdf.savefig()

            plt.close()
	




 




import os
import json
import time

import matplotlib.pyplot as plt

from gonioanalysis.directories import PROCESSING_TEMPDIR
from gonioanalysis.drosox.loading import load_data
from gonioanalysis.binary_search import (
        binary_search_middle,
        binary_search_left,
        binary_search_right
)
from gonioanalysis.image_tools import ImageShower



class XAnalyser:
    '''
    Analysing and getting the analysed results out
    '''
    def __init__(self, data_path, folder):
        '''
        data_path       Path to the data folders
        folder          Name of the DrosoX folder
        '''
        self.data_path = data_path
        self.folder = folder
        self.fly = folder
        
        # Move these away from here
        self.males = ['DrosoX6', 'DrosoX7', 'DrosoX8', 'DrosoX9', 'DrosoX15']
        self.females = ['DrosoX10', 'DrosoX11', 'DrosoX12', 'DrosoX13', 'DrosoX14']
        self.skip_flies = ['DrosoX14']
        
        # Set saving directories and create them
        self.savedirs = {'overlap': 'binocular_overlap', 'level': 'vertical_correction'}
        for key in self.savedirs:
            self.savedirs[key] = os.path.join(PROCESSING_TEMPDIR,
                    'XAnalyser_data',
                    self.savedirs[key])
            os.makedirs(self.savedirs[key], exist_ok=True)
    
        
        print('Initializing XAnalyser, datapath {}, folder {}'.format(data_path, folder))
    

    def get_data(self):
        '''
        Calls drosox.loading.load_data for this fly.
        '''
        return load_data(os.path.join(self.data_path, self.folder))
    

    def measure_overlap(self):
        '''
        Analyses binocular overlap by the binary search method, where
        the user makes decisions wheter the both pseudopupils are visible
        or not.
        '''
        start_time = time.time()
        
        data = load_data(os.path.join(self.data_path, self.folder))

        fig, ax = plt.subplots()
        shower = ImageShower(fig, ax)

        # Try to open if any previously analysed data
        analysed_data = []
        try: 
            with open(os.path.join(PROCESSING_TEMPDIR, 'binary_search', 'results_{}.json'.format(fly)), 'r') as fp:
                analysed_data = json.load(fp)
        except:
            pass
        analysed_pitches = [item['pitch'] for item in analysed_data]
       
        print('Found {} pitches of previously analysed data'.format(len(analysed_data)))

        for i, (pitch, hor_im) in enumerate(data):
            
            # Skip if this pitch is already analysed
            if pitch in analysed_pitches:
                continue

            horizontals = [x[0] for x in hor_im]
            images = [x[1] for x in hor_im]

            N = len(images)
            shower.setImages(images)

            # Ask user to determine middle, left, and right
            i_m = binary_search_middle(N, shower)
            if i_m == 'skip':
                continue

            i_l = binary_search_left(N, shower, i_m)
            i_r = binary_search_right(N, shower, i_m)
            
            analysed_data.append({})

            analysed_data[-1]['N_images'] = N
            analysed_data[-1]['pitch'] = pitch
            analysed_data[-1]['horizontals']= horizontals
            analysed_data[-1]['image_fns']= images
           
            analysed_data[-1]['index_middle'] = i_m
            analysed_data[-1]['index_left'] = i_l
            analysed_data[-1]['index_right']= i_r
            
            analysed_data[-1]['horizontal_middle'] = horizontals[i_m]
            analysed_data[-1]['horizontal_left'] = horizontals[i_l]
            analysed_data[-1]['horizontal_right']= horizontals[i_r]
            

            print('Done {}/{} in time {} minutes'.format(i+1, len(data), int((time.time()-start_time)/60) ))
            
            # Save on every round to avoid work loss
            with open(os.path.join(self.savedirs['overlap'],
                'results_{}.json'.format(self.fly)), 'w') as fp:
                json.dump(analysed_data, fp)


    
    def get_overlaps(self, correct_antenna_level=True):
        '''
        Load the results of binary search.
        
        correct_antenna_level       Corrects with antenna level
        '''
        fn = os.path.join(self.savedirs['overlap'], 'results_{}.json'.format(self.fly))
        
        
        with open(fn, 'r') as fp:
            overlap_markings = json.load(fp)
        

        if correct_antenna_level:
            antenna_level = self.get_antenna_level()

            for i in range(len(overlap_markings)):
                overlap_markings[i]['pitch'] -= antenna_level


        return overlap_markings
    


    def get_antenna_level(self):
        '''
        Load pitch points where the pseudopupils align with antenna.
        Returns Fales if no antenna level for the specimen.

        Run antenna_levels.py first to find antenna levels.
        '''
        fn = os.path.join(self.savedirs['level'], '{}.txt'.format(self.fly))
        
        if os.path.exists(fn):
            with open(fn, 'r') as fp:
                antenna_level = float(fp.read())
            
            return antenna_level
        else:
            #raise OSError('Cannot find antenna level corretion {}'.format(fn))
            return 0.

    def print_overlap(self):
        for d in self.get_overlaps():
            overlap = abs(d['horizontal_right']-d['horizontal_left'])
            line = 'Vertical {} deg: overlap width {} deg'.format(d['pitch'],
                overlap)

            print(line)




import os
import datetime

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

from gonioanalysis.image_tools import open_adjusted
from gonioanalysis.directories import ANALYSES_SAVEDIR


def _make_figure(manalyser, n_rows, i_pg):
    i_pg += 1

    height_ratios = [1 for i in range(n_rows)]
    if i_pg == 0:
        height_ratios[0] *= 1.4

    fig, axes = plt.subplots(n_rows ,3, figsize=(8.27, 11.69),
            gridspec_kw={'width_ratios': [0.618, 1, 1], 'height_ratios': height_ratios})

    fig.suptitle( "{}, page {}".format(manalyser.get_specimen_name(), i_pg))
    
    return fig, axes, i_pg


def _imshow(image, ax):
    ax.imshow(image, cmap='gray')
    ax.set_axis_off()


def _image_folder_info(manalyser, image_folder, ax):
    
    pass

def _specimen_info(manalyser, ax):
    
    string = ""
    
    string += manalyser.get_specimen_sex() + '\n'
    string += manalyser.get_specimen_age() + '\n'


    ax.text()


def pdf_summary(manalysers):
    '''
    Make a structured PDF plotting all the DPP data of
    the fly and any linked data as well.
    '''
    

    pdf_savedir = os.path.join(ANALYSES_SAVEDIR, 'reports')
    os.makedirs(pdf_savedir, exist_ok=True)

    # Subplot rows per page
    n_rows = 4

    with PdfPages(os.path.join(pdf_savedir, 'pdf_summary_{}.pdf'.format(datetime.datetime.now()))) as pdf:
    
        for manalyser in manalysers:
            
            # The page index for this fly
            i_pg = -1
            fig, axes, i_pg = _make_figure(manalyser, n_rows, i_pg)

            # SNAP / FACE IMAGE
            snap_fn = manalyser.get_snap_fn()
            
            if snap_fn:
                face = open_adjusted( manalyser.get_snap_fn() )
                _imshow(face, axes[0][2])

            # Information about the fly
            
            # DPP data
            for i, image_folder in enumerate(manalyser.list_imagefolders()):
                
                if i+1 >= n_rows:
                    plt.tight_layout()
                    pdf.savefig()
                    plt.close()

                    fig, axes, i_pg = _make_figure(manalyser, n_rows, i_pg)
                
                i_row = i+1 - i_pg*n_rows
                
                axes[i_row][1].set_title(image_folder)

                # Photo of the location
                location_im = open_adjusted( manalyser.list_images(image_folder, absolute_path=True)[0] )
                axes[i_row][0].imshow(location_im, cmap='gray')
                _imshow(location_im, axes[i_row][0])

                displacements = manalyser.get_displacements_from_folder(image_folder)
                for dis in displacements:
                    axes[i_row][1].plot(dis, lw=1)

                axes[i_row][1].plot(np.mean(displacements, axis=0), lw=3, color='black')
            

            plt.tight_layout()
            pdf.savefig()
            plt.close()








import gonioanalysis.settings as settings

DEFAULT_SAVEFN = 'gonioanalysis-tkgui-settings.json'

def set(*args, fn=DEFAULT_SAVEFN, **kwargs):
    return settings.set(*args, fn=fn, **kwargs)

def get(*args, fn=DEFAULT_SAVEFN, **kwargs):
    return settings.get(*args, fn=fn, **kwargs)




'''
Functions related to DrosoM data loading.

MODULE LEVEL VARIABLES
----------------------
REPETITION_INDICATOR : str
    In filenames, the text preceding the repetition value
POSITION_INDICATOR : str
    In filenames, the text preceding the imaging location value
IMAGE_NAME_EXTENSIONS : str
    File name extensions that are treated as image files.
'''

import os
import ast

from gonioanalysis.rotary_encoders import to_degrees


REPETITION_INDICATOR  = 'rep'
POSITION_INDICATOR = 'pos'
CAMERA_INDICATOR = '_cam{}'

IMAGE_NAME_EXTENSIONS = ('.tiff', '.tif')


def arange_fns(fns):
    '''
    Arange filenames based on REPETITION_INDICATOR and POSITION_INDICATOR
    in their time order (repeat 1, image1,2,3,4, repeat 2, image1,2,3,4, ...).
    
    If no indicators are found in the filenames then the ordering is
    at least alphabetical.
    '''

    # Sort by i_frame
    try:
        fns.sort(key=lambda x: int(x.split('_')[-1].split('.')[0]))
    except ValueError:
        # Here if image fn not enging with _somenumber.tif(f)
        pass
    
    # Sort by i_repeat
    try:
        fns.sort(key=lambda x: int(x.split('_')[-2][3:]))
    except ValueError:
        fns.sort()

    return fns


def split_to_repeats(fns):
    '''
    Split a list of filenames into repeats (sublists) based on the
    REPETITION_INDICATOR

    Arguments
    ---------
    fns : list
        1D sequence of filenames

    Returns
    --------
    splitted_fns : list
        A list where each item is a sublist of filenames (or empty list
        if there were no filenames for that repeat)
    '''
    
    repeats = {}
    
    for fn in fns:
        try:
            i_repeat = str(int(fn[fn.index(REPETITION_INDICATOR)+len(REPETITION_INDICATOR):].split('_')[0]))
        except ValueError:
            print('Warning: Cannot determine i_repeat for {}'.format(fn))
        
        if i_repeat not in repeats:
            repeats[i_repeat] = []
        
        repeats[i_repeat].append(fn)
    
    return [fns for i_repeat, fns in repeats.items()]



def angleFromFn(fn):
    '''
    Returns the horizontal and vertical angles from a given filename
    The filename must be IMSOFT formatted as
        im_pos(-30, 170)_rep0_0.tiff

    fn          Filename, from which to read the angles
    '''
    hor, ver = fn.split('(')[1].split(')')[0].split(',')
    hor = int(hor)
    ver = int(ver)
    
    angles = [[hor,ver]]
    to_degrees(angles)
    return angles[0]


def angles_from_fn(fn, prefix='pos'):
    '''
    Takes in a filename that somewhere contains string "pos(hor, ver)",
    for example "pos(-30, 170)" and returns tuple (-30, 170)
    
    Returns
    -------
    angle : tuple of ints
        Rotation stage values or (0, 0) if the rotation was not found.
    '''
    try:
        i_start = fn.index(prefix) + len(prefix)
    except ValueError:
        #raise ValueError("Cannot find prefix {} from filename {}".format(fn))
        return (0,0)
    try:
        i_end = fn[i_start:].index(')') + i_start + 1
    except ValueError:
        #raise ValueError("Cannot find ')' after 'pos' in filename {}".format(fn))
        return (0,0)
    return ast.literal_eval(fn[i_start:i_end])



def load_data(drosom_folder):
    '''Loads GonioImsoft imaging data filenames into a dictionary.

    The data has to be saved according to the folder hierarchy:

        LEVEL   Content             Examples
        1       specimen_folder     "DrosoM2"
        2       image_folder        "pos(0, 0)" "pos(20, 20)") "pos(0, 10)_somesuffixhere"
        3       image files         "im_pos(0, 0)_rep0_stack.tiff", "im_pos(0, 0)_rep0_cam2_stack.tiff"

    Arguments
    ---------
    drosom_folder : string
        Path to the specimen folder that contains the image folders.
    
    Returns
    -------
    stacks_dictionary : dict
        Contains the str((horizontal, pitch)) as keys and a list of image stacks
        as values.
        
            stacks_dictionary = {"(hor1, pitch1): [[stack_rep1], [stack_rep2], ...]"},
            where stack_rep1 = [image1_fn, image2_fn, ...].
    
        Notice 1: The horizontal and pitch angles in the keys are in
        rotary encoder steps (not degrees).
    
    '''
    
    stacks_dictionary = {}

    pos_folders = [fn for fn in os.listdir(drosom_folder) if os.path.isdir(os.path.join(drosom_folder, fn))]

    # Import all tif images
    for folder in pos_folders:

        if folder == 'snaps':
            continue
        
        if folder.startswith(POSITION_INDICATOR):
            str_angles = folder[len(POSITION_INDICATOR):]     # Should go from "pos(0, 0)" to "(0, 0)"
        else:
            str_angles = folder

        files = os.listdir(os.path.join(drosom_folder, folder))
        tiff_files = [f for f in files if f.endswith(IMAGE_NAME_EXTENSIONS)]
        
        if len(tiff_files) == 0:
            # Skip if no images in the folder
            continue

        # Detect if many cameras have been used ("_cam{i}_" in any image filenames)      
        cameras = {}
        for i_camera in range(10):
            indicator = CAMERA_INDICATOR.format(i_camera)
            matching = [fn for fn in tiff_files if indicator in fn]
            if matching:
                cameras[f'_cam{i_camera}'] = matching

        if not cameras:
            cameras = {'': tiff_files}

        
        for camera, images in cameras.items():

            cameras_images = arange_fns(images)
            key = str_angles + camera
            stacks_dictionary[key] = []

            # Subdivide into repetitions
            for tiff in cameras_images:
                try:
                    i_repetition = int(tiff[tiff.index(REPETITION_INDICATOR)+len(REPETITION_INDICATOR):].split('_')[0])
                except ValueError:
                    print('Warning: Cannot determine i_repetition for {}'.format(tiff))
                    i_repetition = 0

                while i_repetition >= len(stacks_dictionary[str_angles+camera]):
                    stacks_dictionary[key].append([])
                
                stacks_dictionary[key][i_repetition].append(os.path.join(drosom_folder, folder, tiff))
            
            # Remove empty lists, if one repetition index or more is missing from the data
            stacks_dictionary[key] = [alist for alist in stacks_dictionary[key] if not alist == []]

    return stacks_dictionary




'''
binary_search.py
Determining binocular overlap by half-interval search.

DESCRIPTION


TODO
- examination mode, fine tune changes
- combine binarySearchs to a single function for easier mainantance
- remove reverse option from binarySearch

'''
import os
import json
import time
from math import floor

import matplotlib.pyplot as plt

from gonioanalysis.directories import ANALYSES_SAVEDIR
from gonioanalysis.droso import DrosoSelect
from gonioanalysis.image_tools import ImageShower


def inputRead():
    while True:
        try:
            inpt = input('>> ')
            if inpt == 's':
                return 'skip'
            direction = int(inpt)
            break
        except ValueError:
            pass
    return direction

def calcM(L, R):
    '''
    Calculate m used in the binary search.
    '''
    return int(floor(abs(L+R)/2))


def binary_search_middle(N_images, shower, reverse=False):
    '''
    Search for the border where pseudopupils are visible simultaneously.
    Midpoint search.

    shower UI class

    '''
    DIR = [1,2]
    if reverse:
        DIR = [2,1]


    right_end = N_images-1

    R = right_end
    L = 0
    
    #print('Find midpoint. "1" to go left or "2" to go right')
    print('Binary search')
    print('  Type in 1 or 2 to rotate\n  -1 to return to the beginning\n  0 to instaselect')
    
    shower.setTitle('Midpoint')

    while L<R:
        
        m = calcM(L, R)
        shower.setImage(m)
        shower.cacheImage(calcM(L, m-1))
        shower.cacheImage(calcM(m+1, R))


        direction = inputRead()
        
        if direction == 'skip':
            return direction

        if direction == DIR[0]:
            R = m-1
        elif direction == DIR[1]:
            L = m+1
        elif direction == 0:
            break

        elif direction == -1:
            R = right_end
            L = 0
 
    return int((R+L)/2)


def binary_search_left(N_images, shower, midpoint, reverse=False):
    '''
    Search for the border where pseudopupils are visible simultaneously.
    Left side search.

    shower UI class

    '''
    DIR = [1,2]
    if reverse:
        DIR = [2,1]


    R = midpoint
    L = 0

    print('How many pupils (1 or 2)?')
    shower.setTitle('Left side')


    while L<R:
        m = calcM(L, R)
        shower.setImage(m)
        shower.cacheImage(calcM(L, m-1))
        shower.cacheImage(calcM(m+1, R))

       
        N_pupils = inputRead()

        if N_pupils == DIR[1]:
            R = m-1
        elif N_pupils == DIR[0]:
            L = m+1
            
        elif N_pupils == -1:
            R = midpoint
            L = 0

    
    return int((R+L)/2)


def binary_search_right(N_images, shower, midpoint, reverse=False):
    '''
    Search for the border where pseudopupils are visible simultaneously.
    Right side search.

    shower UI class

    '''
    DIR = [1,2]
    if reverse:
        DIR = [2,1]

    right_end = N_images-1

    R = right_end
    L = midpoint
    
    print('How many pupils (1 or 2)?')
    shower.setTitle('Right side')


    while L<R:
        m = calcM(L, R)
        shower.setImage(m)
        shower.cacheImage(calcM(L, m-1))
        shower.cacheImage(calcM(m+1, R))

        N_pupils = inputRead()

        if N_pupils == DIR[1]:
            L = m+1

        elif N_pupils == DIR[0]:
            R = m-1
            
        elif N_pupils == -1:
            R = right_end
            L = midpoint
 
    return int((R+L)/2)



def main():
    start_time = time.time()
    

    # Letting user to select a Droso folder
    selector = DrosoSelect()
    folder = selector.askUser()
    fly = os.path.split(folder)[1]


    xloader = XLoader()
    data = xloader.getData(folder)

    fig, ax = plt.subplots()
    shower = ImageShower(fig, ax)



    # Try to open if any previously analysed data
    analysed_data = []
    try: 
        with open(os.path.join(ANALYSES_SAVEDIR, 'binary_search', 'results_{}.json'.format(fly)), 'r') as fp:
            analysed_data = json.load(fp)
    except:
        pass
    analysed_pitches = [item['pitch'] for item in analysed_data]
   
    print('Found {} pitches of previously analysed data'.format(len(analysed_data)))



    for i, (pitch, hor_im) in enumerate(data):
        
        # Skip if this pitch is already analysed
        if pitch in analysed_pitches:
            continue
        

        horizontals = [x[0] for x in hor_im]
        images = [x[1] for x in hor_im]

        N = len(images)
        shower.setImages(images)



        # Ask user to determine middle, left, and right
        i_m = binarySearchMiddle(N, shower)
        if i_m == 'skip':
            continue

        i_l = binarySearchLeft(N, shower, i_m)
        i_r = binarySearchRight(N, shower, i_m)
        
        analysed_data.append({})

        analysed_data[-1]['N_images'] = N
        analysed_data[-1]['pitch'] = pitch
        analysed_data[-1]['horizontals']= horizontals
        analysed_data[-1]['image_fns']= images
       
        analysed_data[-1]['index_middle'] = i_m
        analysed_data[-1]['index_left'] = i_l
        analysed_data[-1]['index_right']= i_r
        
        analysed_data[-1]['horizontal_middle'] = horizontals[i_m]
        analysed_data[-1]['horizontal_left'] = horizontals[i_l]
        analysed_data[-1]['horizontal_right']= horizontals[i_r]
        


        print('Done {}/{} in time {} minutes'.format(i+1, len(data), int((time.time()-start_time)/60) ))
        
        # Save on every round
        with open(os.path.join(ANALYSES_SAVEDIR, 'binary_search/', 'results_{}.json'.format(fly)), 'w') as fp:
            json.dump(analysed_data, fp)


if __name__ == "__main__":
    main()
    





import numpy as np

from gonioanalysis.coordinates import force_to_tplane, normalize

def get_reference_vector(P0):
    '''
    Generate a reference vector for the coffiecient of determination
    (R**2 value) calculation.

    The reference vectors are always in the XY-plane and the sphere's
    tangential plane pointing east.
    
    Returns vec so that P0+vec=P1
    '''
    px,py,pz = P0
    aP0 = np.array(P0)
    
    if px==0:
        if py > 0:
            phi = np.pi/2
        else:
            phi = -np.pi/2

    else:
        phi = np.arctan(py/px)
    
    if px < 0:
        phi += np.pi

    # x,y,z components
    vx = -np.sin(phi)
    vy = np.cos(phi)
    
    vec = force_to_tplane(aP0, aP0+np.array((vx,vy,0)))
    vec = normalize(aP0, vec, scale=.15)
    vec = vec - aP0

    return vec
    
        





'''
Gonio-analysis tkinter GUI widgets.
'''
import os
import numpy as np

import tkinter as tk
from tkinter import filedialog

from tk_steroids.elements import (
        ButtonsFrame,
        TickboxFrame,
        )
from tk_steroids.matplotlib import CanvasPlotter
from tk_steroids.routines import inspect_booleans
from tk_steroids.menumaker import MenuMaker

from gonioanalysis.drosom.analysing import MAverager
from gonioanalysis.drosom.plotting.basics import (
        plot_3d_vectormap,
        plot_3d_differencemap,
        )
from .common import select_specimens


class RotationButtons(tk.Frame):
    '''
    Create buttons to set a matplotlib 3D plot rotations
    
    Attrubutes
    ----------
    axes : list
        Associated matplotlib axes that get rotated
    buttons_frame : object
        Buttons frame object containing the tkinter under the buttons attribute
    rotation_offset : tuple
        (elev, azim) offset in rotations
    callback : callable or None
        Additional callback to be called after changing rotation.
    '''

    def __init__(self, tk_parent, axes, rotations, callback=None,
            label='', hide_none=True, rotation_offset=(0,0)):
        '''
        tk_parent : object
            Tkinter parent object
        axes : list
            List of matplotlib axes
        rotations : list of tuples
            Rotations [(elev, azim), ...]. If any None, keeps the corresponding
            rotation as it is.
        callback : None or callable
            Callback after each rotation update
        hide_none : bool
            When one of the rotations is None, hide the None from
            button text
        rotation_offset : tuple
            Offset in elevation and azitmuth, respectively in degrees
        '''
        tk.Frame.__init__(self, tk_parent)
        
        self.axes = axes
        self.rotation_offset = rotation_offset
        
        if hide_none:
            names = []
            for rotation in rotations:
                if None in rotation:
                    for r in rotation:
                        if r is not None:
                            names.append(r)
                else:
                    names.append(rotation)
        else:
            names = rotations

        commands = [lambda rot=rot: self.set_rotation(*rot) for rot in rotations]
        self.buttons_frame = ButtonsFrame(self, names, commands, label=label)
        self.buttons_frame.grid(row=1, column=2)
       
        self.callback = callback


    def set_rotation(self, elev, azim):
        for ax in self.axes:
            if elev is None:
                uelev = ax.elev
            else:
                uelev = elev + self.rotation_offset[0]

            if azim is None:
                uazim = ax.azim
            else:
                uazim = azim + self.rotation_offset[1]

            ax.view_init(uelev, uazim)

        if callable(self.callback):
            self.callback()



class CompareVectormaps(tk.Frame):
    '''
    Widget to compare two vectormaps interactively to
    each other.

    Attributes
    ----------
    tk_parent : object
        Parent widget
    plot_functions : list
        List of plot functions,
        default [plot_3d_vectormap, plot_3d_vectormap, plot_3d_differencemap]
    canvases : list
        CanvasPlotter objects, canvas2 to show difference
    tickbox_frames : list
        List of tickbox frame objects
    buttons : list
        Corresponding tk.Buttons under the canvases
    analysers : list
        Analyser objects selected by the user (len == 2)
    '''


    def __init__(self, tk_parent, core):
        class FileMenu(MenuMaker):
        
            def __init__(self, main_widget, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.main_widget = main_widget
            
            def save_all_views(self):
                self.main_widget.savefig()

            def close_window(self):
                self.main_widget.tk_parent.destroy()
                
        tk.Frame.__init__(self, tk_parent)
        self.tk_parent = tk_parent
        
        self.core = core

        self.plot_functions = [plot_3d_vectormap, plot_3d_vectormap,
                plot_3d_differencemap]
        self.canvases = []
        self.tickbox_frames = []
        self.buttons = []
        self.analysers = [None, None]

        self.grid_rowconfigure(0, weight=1)

        axes = []
        for i in range(3):
            canvas = CanvasPlotter(self, projection='3d')
            canvas.grid(row=3, column=i, sticky='NSWE')
            self.canvases.append(canvas)

            axes.append(canvas.ax)
            axes[-1].elev = 15
            axes[-1].azim = 60
            
            # Plot settings
            if i in [0,1]:
                cmd = lambda i=i: self.set_vectormap(i_canvas=i)
            else:
                cmd = self.plot_difference
            options, defaults = inspect_booleans(self.plot_functions[i])
            tickboxes = TickboxFrame(self, options, defaults=defaults,
                    callback=cmd)
            tickboxes.grid(row=4, column=i, sticky='NSWE')
            self.tickbox_frames.append(tickboxes)

            # Main buttons
            if i in [0, 1]:
                cmd = lambda i=i: self.select_specimens(i_canvas=i)
                txt = 'Select specimens...'
            else:
                cmd = self.plot_difference
                txt = 'Compare'

            button = tk.Button(self, text=txt, command=cmd)
            button.grid(row=45, column=i)
            self.buttons.append(button)

            self.grid_columnconfigure(i, weight=1)

        
        hors = [-80, -60, -50, -30, -15, 0, 30, 15, 50, 60, 80]
        verts = hors
        
        hors = [(None, hor) for hor in hors]
        verts = [(ver, None) for ver in verts]

        for i, (name, rotations) in enumerate(zip(['Horizontal', 'Vertical'], [hors, verts])):
            self.rotation_buttons = RotationButtons(self, axes, rotations,
                    label=name+' rotation', callback=self._update_canvases,
                    rotation_offset=(0,90))
            self.rotation_buttons.grid(row=i+1, column=0, columnspan=3)

        self.menubar = tk.Menu()
        self.filemenu = FileMenu(self, 'File')
        self.filemenu._connect(self.menubar)
        self.winfo_toplevel().config(menu=self.menubar)


    def _update_canvases(self):
        for i in range(3):
            ax = self.canvases[i].ax
            #if ax.dist != 8.5:
            #    ax.dist = 8.5
 
            self.canvases[i].update()

    def select_specimens(self, i_canvas):
        select_specimens(self.core, self.set_vectormap,
                command_args=[i_canvas], return_manalysers=True,
                with_movements=True)


    def set_vectormap(self, manalysers=None, i_canvas=None):
        import time 
        start_time = time.time()

        canvas = self.canvases[i_canvas]
        ax = canvas.ax
        
        if manalysers is None:
            analyser = self.analysers[i_canvas]
            if analyser is None:
                return None
        else:
            if len(manalysers) > 1:
                analyser = MAverager(manalysers)
            else:
                analyser = manalysers[0]
        
        azim, elev = (ax.azim, ax.elev)
        ax.clear()
        
       
        kwargs = self.tickbox_frames[i_canvas].states
        
        plot_3d_vectormap(analyser, ax=ax, azim=azim, elev=elev,
                mutation_scale=6, scale_length=1.2, **kwargs)
       
        #if ax.dist != 8.5:
        #    ax.dist = 8.5
 
        canvas.update()

        self.analysers[i_canvas] = analyser

        print('took {} seconds'.format(time.time()-start_time))


    def plot_difference(self):
        
        if any([an is None for an in self.analysers]):
            return None
        
        kwargs = self.tickbox_frames[-1].states
        
        ax = self.canvases[-1].ax
        ax.clear()
        plot_3d_differencemap(*self.analysers[0:2], ax=ax, **kwargs)
        
        #if ax.dist != 8.5:
        #    ax.dist = 8.5

        self.canvases[-1].update()

    
    def savefig(self, i_canvas=None, fn=None):
        '''
        Save current images visible on the canvases

        i_canvas : int or None
            If none, save all views by inserting index of the canvas
            to the end of the saved filename.
        fn : string or None
            If not given, save name is asked from the user.
        '''
        
        if i_canvas is None:
            iterate = range(len(self.canvases))
        elif isinstance(i_canvas, int) and i_canvas:
            iterate = [i_canvas]
        else:
            raise ValueError('wrong type for i_canvas: {}'.format(i_canvas))

        if fn is None:
            
            if i_canvas == 'all':
                text = 'Select common save name for the views'
            else:
                text = 'Save image on a view'

            fn = filedialog.asksaveasfilename(title=text)
            
            if not '.' in os.path.basename(fn):
                fn = fn + '.png'

            if fn:
                for i_canvas in iterate:
                    efn = '.'.join(fn.split('.')[:-1]+[str(i_canvas)]+fn.split('.')[-1:])
                    self.canvases[i_canvas].figure.savefig(efn, dpi=600)



'''
Central settings for save/load directories.
'''

import os
import platform

CODE_ROOTDIR = os.path.dirname(os.path.realpath(__file__))
USER_HOMEDIR = os.path.expanduser('~')

if platform.system() == "Windows":
    GONIODIR = os.path.join(USER_HOMEDIR, 'GonioAnalysis')
else:
    GONIODIR = os.path.join(USER_HOMEDIR, '.gonioanalysis')


ANALYSES_SAVEDIR = os.path.join(GONIODIR, 'final_results')
PROCESSING_TEMPDIR = os.path.join(GONIODIR, 'intermediate_data')
PROCESSING_TEMPDIR_BIGFILES = os.path.join(GONIODIR, 'intermediate_bigfiles')


# DIRECTORIES THAT HAVE TO BE CREATED
ALLDIRS= {
        'GONIODIR': GONIODIR,
        'ANALYSES_SAVEDIR': ANALYSES_SAVEDIR,
        'PROCESSING_TEMPDIR': PROCESSING_TEMPDIR,
        'PROCESSING_TEMPDIR_BIGFILES': PROCESSING_TEMPDIR_BIGFILES
        }


def print_directories():

    print('These are the directories')
    
    for key, item in ALLDIRS.items():
        print('{} {}'.format(key, item))



def cli_ask_creation(needed_directories):
    '''
    Short command line yes/no.
    '''

    # Temporary fix; Launching GUI for the first time fails
    # if directories not present so always make them
    return True

    print("The following directories have to be created")

    for directory in needed_directories:
        print("  {}".format(directory))

    print("\nIs this okay? (yes/no)")

    while True:
        selection = input(">> ").lower()

        if selection == "yes":
            return True
        elif selection == "no":
            return False
        else:
            print("Choice not understood, please type yes or no")



def directories_check(ui=cli_ask_creation):
    '''Checks if the save directories exists, and creates if not.
    
    ARGUMENTS
    ---------
    ui : None or callable
        A callable that returns True if selects yes (wants to create the
        directories or False if not. The callable receives a list as its
        first (and only) argument that contains the to-be-created
        directories.
    '''
    non_existant = []
    
    for key, item in ALLDIRS.items():
        if not os.path.exists(item):
            non_existant.append(item)

    # If some directories are not created, launch an input requiring
    # user interaction.
    if non_existant:
        if callable(ui):
            if ui(non_existant) == True:
                for directory in non_existant:
                    os.makedirs(directory, exist_ok=True)
            else:
                raise NotImplementedError("Reselecting directories in UI not yet implemented")
        else:
            raise OSError("All directories not created and ui is not callable")



if __name__ == "__main__":
    print_directories()
else:
    directories_check()







import numpy as np
import matplotlib.pyplot as plt

from gonioanalysis.droso import simple_select


def cli_group_and_compare(manalysers):
    '''
    Command line user interface to create grouped data for
    compare_before_after() function.
    '''
    
    grouped_data = []

    print('For compare_before_after function, we have to group the data')

    for i_manalyser, manalyser in enumerate(manalysers):
        print('Specimen {}/{}'.format(i_manalyser+1, manalyser))


        image_folders = manalyser.list_imagefolders()
        image_folders.sort()
    
        for i_pair in range(0, int(len(image_folders)/2)):
            
            print('Select a BEFORE experiment')
            be = simple_select(image_folders)
            
            image_folders.remove(be)

            print('Select an AFTER experiment')
            af = simple_select(image_folders)
            
            image_folders.remove(af)

            grouped_data.append([manalyser, be, af])


    compare_before_after(grouped_data)


def compare_before_after(grouped):
    '''
    
    Grouped:   List where each element is
            [manalyser, image_folder_before, image_folder_after]

    '''

    fig, axes = plt.subplots(1, 2, sharey=True, sharex=True)
    

        
    colors = plt.cm.get_cmap('Dark2', len(grouped))


    # if stands for image_folder
    for i, (manalyser, if_before, if_after) in enumerate(grouped):
        
        be = manalyser.get_displacements_from_folder(if_before)
        af = manalyser.get_displacements_from_folder(if_after)
        
        mbe = np.mean(be, axis=0)
        maf = np.mean(af, axis=0)
        
        
        axes[0].plot(mbe, color=colors(i))
        axes[1].plot(maf, color=colors(i))


    axes[0].set_title('Before')
    axes[1].set_title('After')
    









import os
import cv2
from multiprocessing import Pool
import tempfile

import tifffile

from videowrapper import VideoWrapper
from gonioanalysis.drosom.loading import arange_fns, split_to_repeats


datadir = input("DATADIR TO COMPRESS (av1 encode): ")
newdir = datadir+'_av1compressed'
print("New data will be saved in {}".format(newdir))
os.makedirs(newdir, exist_ok=True)


def _expand_stack(fn, tmp_path):
    '''
    Expand a tiff stack to separate tiff files using tifffile.
    '''
    stack = tifffile.TiffFile(fn)

    for i_page in range(len(stack.pages)):
        image = stack.asarray(key=i_page)
        
        newfn = os.path.join(tmp_path, fn+str(i_page).zfill(8)+'.tiff'
        tifffile.imsave(newfn, image)

    return tmp_fns


def _clear_tmp(tmp_fns):
    for fn in tmp_fns:
        os.remove(fn)


def process_dir(root):
    directory = root
    
    fns = [fn for fn in os.listdir(directory) if fn.endswith(('.tiff', '.tif'))]
    fns = arange_fns(fns)
    fns = split_to_repeats(fns)
    
    tmpdir = tmpfile.TemporaryDirectory()
    
    for repeat in fns:
        common = os.path.commonprefix(repeat)
        savedir = root.replace(os.path.basename(datadir), os.path.basename(newdir))
        
        if 'stack' in common.lower():
            tmpdir.cleanup()
            if len(repeat) != 1:
                raise NotImplementedError('One repeat, multiple stacks, not implemented')
            _expand_stack(repeat, tmpdir.name)

            location = tmpdir.name
            movie_fn = os.path.join(savedir, common)
        else:
            location = root
            movie_fn = os.path.join(savedir, common+'stack.mp4')
        
        if os.path.exists(movie_fn):
            continue
        
        os.makedirs(savedir, exist_ok=True)
        video = VideoWrapper()
        video.images_to_video([os.path.join(location, fn) for fn in repeat], movie_fn, 1)
        

def main():

    tiffdirs = []
    for root, dirs, fns in os.walk(datadir):
        
        if os.path.basename(root) == "snaps":
            continue
        
        hastiffs = any([fn.endswith('.tiff') for fn in fns])
        
        if hastiffs:
            tiffdirs.append(root)
            
    with Pool() as p:
        p.map(process_dir, tiffdirs)

if __name__ == "__main__":
    main()



'''
Transforming rotary encoder step values into degrees and vice versa.
'''

DEFAULT_STEPS_PER_REVOLUTION = 1024 


def to_degrees(angle_pairs):
    '''
    Transform angle pairs (paired the steps of rotary encoder)
    to corresponding degree angle values in place.

    angle_pairs     List of (horizontal, vertical)
    '''

    for i in range(len(angle_pairs)):
        angle_pairs[i][0] *= (360/1024)
        angle_pairs[i][1] *= (360/1024)


def step2degree(step, steps_per_revolution=DEFAULT_STEPS_PER_REVOLUTION):
    '''
    Transform a rotary encoder step count (such 54 steps) into corresponding
    rotation in degrees (54 * steps_per_revolution).
    '''
    return step * (360/steps_per_revolution)


def degree2step(angle, steps_per_revolution=DEFAULT_STEPS_PER_REVOLUTION):
    '''
    Transform a rotation angle from degrees to corresponging rotary encoder steps.
    '''
    return angle * (steps_per_revolution/360)





'''
Simple tool (Encoder) to transform an image recording into a movie clip.
'''

import os
import datetime
import subprocess

PROCESSING_TEMPDIR = '/tmp/videowrapper_tmp'


class VideoWrapper:
    '''
    Simple Python interface for encoding images into a movie clip
    using ffmpeg (required to be installed).

    See Encoder.encode for usage.
    '''

    def __init__(self):
        
        # Ensure that ffmpeg is installed
        try:
            subprocess.check_output('ffmpeg -version', shell=True)
        except subprocess.CalledProcessError:
            raise RuntimeError('ffmpeg is not installed (required by movie.Encoder)')
        
        # Temporal stuff directory
        self.tempdir = os.path.join(PROCESSING_TEMPDIR, 'movie-encoder')
        os.makedirs(self.tempdir, exist_ok=True)


    def _makeImageList(self, images, fps):
        '''
        PRIVATE Used from self.encode.
        
        Creates a textfile that contains all the image filenames and is passed to ffmpeg.
        '''
        imagelist_fn = os.path.join(self.tempdir, 'imagelist_{}.txt'.format(str(datetime.datetime.now()).replace(' ', '_')))
        imagelist_fn = os.path.abspath(imagelist_fn)
        with open(imagelist_fn, 'w') as fp:
            for image in images:
                fp.write("file '"+os.path.abspath(image)+"'\n")
                fp.write('duration {}\n'.format(float(1/fps)))
        
        return imagelist_fn


    def images_to_video(self, images, output_fn, fps, codec='libaom-av1'):
        '''
        Encoding a video clip using ffmpeg.

        INPUT ARGUMENTS         DESCRIPTION
        images                  Ordered list of images; Movie will be encoded in this order.
        output_fn               Output name of the movie.
        fps                     Desrided frames per second.

        Encoding libx264, preset veryslow and crf 28 will be used.
        '''

        # Check that input is correct and that images actually exist (avoids many trouble).
        
        options = {}

        if type(output_fn) != type('string'):
            raise TypeError('output_fn has to be a string (required by movie.Encoder)')

        try:
            float(fps)
        except:
            raise TypeError('fps has to be floatable')
        
        for image in images:
            if not os.path.exists(image):
                raise FileNotFoundError('Precheck in movie.Encoder failed. File {} does not exist'.format(image))
        


        # Check that directory for ouput_fn exsits, if not, create 
        os.makedirs(os.path.dirname(output_fn), exist_ok=True)
        
        imagelist_fn = self._makeImageList(images, fps)
        print(imagelist_fn)
        command = 'ffmpeg -r {} -f concat -safe 0 -i "{}" -c:v {} -pix_fmt gray12 -crf 7 -b:v 0 "{}"'.format(fps,imagelist_fn, codec, output_fn)
        subprocess.check_output(command, shell=True)
        
        os.remove(imagelist_fn)



import os
import math
import multiprocessing as mp

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Arrow, Circle
import tifffile
from scipy.spatial import cKDTree as KDTree
import cv2

from gonioanalysis.directories import ANALYSES_SAVEDIR
import gonioanalysis.coordinates as coordinates
from gonioanalysis.drosom.plotting.common import vector_plot
from gonioanalysis.drosom.loading import angles_from_fn
from gonioanalysis.version import used_scipy_version

def _load_image(fn, roi, e):

    image = tifffile.imread(fn)
    
    x,y,w,h = [int(round(z)) for z in roi]
    
    upper = np.percentile(image[y-e:y+h+e,x-e:x+w+e], 99.5) 
    lower = np.percentile(image[y-e:y+h+e,x-e:x+w+e], 0.5)

    image = np.clip(image, lower, upper) - lower
    image = (image-np.min(image)) / np.max(image)
    image *= 255
    
    return np.array([image,image,image])


def _box(image, roi, lw, color, crop_factor=1):
    x,y,w,h = [int(round(z)) for z in roi]
    
    for i, c in enumerate(color):
        # Top
        image[i, y:y+lw,x:x+w] = c
        # Left
        image[i, y:y+h,x:x+lw] = c
        # Bottom
        image[i, y+h-lw:y+h,x:x+w] = c
        # Right
        image[i, y:y+h,x+w-lw:x+w] = c

    return image

def _crop(image, roi, factor):
    
    x,y,w,h = [int(round(z)) for z in roi]
    cp = [x+int(w/2), y+int(h/2)]

    new_image = []
    
    if factor < 1:
        h2 = int(round(factor*image.shape[1]/2))
        a = cp[1]-h2
        b = cp[1]+h2

        if a < 0:
            a = 0
            b = h2*2
        if b > image.shape[1]:
            a = image.shape[1] - h2*2
            b = image.shape[1]

        for i in range(len(image)):
            new_image.append( image[i, a:b, :] )
    elif factor > 1:
        w2 = int(round(image.shape[2]/2/factor))
        for i in range(len(image)):
            new_image.append( image[i, cp[0]-w2:cp[0]+w2, :] )
    else:
        return image

    return np.array(new_image)





def moving_rois(manalyser, roi_color='red,blue', lw=3, e=50,
        rel_rotation_time=1, crop_factor=0.5,
        _exclude_imagefolders=[], _order=None,
        _draw_arrow=False):
    '''
    Visualization video how the ROI boxes track the analyzed features,
    drawn on top of the original video frames.

    Arguments
    ---------
    roi : string
        A valid matplotlib color. If two comma separated colors
        given use the first for the left eye and the second for the right.
    lw : int
        ROI box line width, in pixels
    e : int
        Extended region for brightness normalization, in pixels
    rel_rotation_time : int or float
        Blend the last and the first next frame for "smooth"
        transition
    crop_factor : int
        If smaller than 1 then cropped in Y.
    
    RETURNS
    -------
    None
    '''
    savedir = os.path.join(ANALYSES_SAVEDIR, 'illustrate_experiments', 'moving_rois', manalyser.get_specimen_name())
    os.makedirs(savedir, exist_ok=True)
    if _draw_arrow:
        os.makedirs(os.path.join(savedir, 'inset'), exist_ok=True)

    colors = roi_color.split(',')
    
    image_fns, ROIs, angles = manalyser.get_time_ordered()

    # ------------------
    # For mosaic
    if _exclude_imagefolders:
        
        newdata = []
        for fn, ROI, angle in zip(image_fns, ROIs, angles):
            if not angles_from_fn(os.path.basename(os.path.dirname(fn))) in _exclude_imagefolders:
                newdata.append([fn, ROI, angle])
            else:
                pass
        image_fns, ROIs, angles = list(zip(*newdata))

    if _order:
        image_fns = list(image_fns)
        ROIs = list(ROIs)
        angles = list(angles)
        newdata = []

        for o in _order:
            
            indices = [i for i in range(len(image_fns)) if angles_from_fn(os.path.basename(os.path.dirname((image_fns[i])))) == o]
            
            for i in indices:
                newdata.append([image_fns[i], ROIs[i], angles[i]])

        image_fns, ROIs, angles = list(zip(*newdata))
    # End for mosaic
    # ------------------

    N = len(image_fns)
    i_frame = 0

    crop_roi = ROIs[0]
    

    if _draw_arrow:
        # Create and setup figure
        fig, ax = plt.subplots(figsize=(10,10))
        ax._myarrow = None
        ax.set_xlim(-1.5, 1.5)
        ax.set_ylim(-1.5, 1.5)
        ax.set_axis_off()
        
        # vector normalisation values
        normalisations = {fol: manalyser.get_displacements_from_folder(fol)[0][-1] for fol in manalyser.list_imagefolders(only_measured=True)}
        
        ax.add_patch( Circle((0,0), 1, fill=False, lw=3, color="gray") )
        

    def draw_arrow_inset(savefn, vx, vy, **kwargs):
        if ax._myarrow:
            ax._myarrow.remove()

        ax._myarrow = Arrow(0,0,vx,vy, width=0.5, **kwargs)
        ax.add_patch(ax._myarrow)
        fig.savefig(savefn, transparent=True)


    for i_fn, (fn, roi, angle) in enumerate(zip(image_fns, ROIs, angles)):

        if i_fn+1 < len(image_fns) and angle != angles[i_fn-1]:
            crop_roi = roi
        
        print("{}/{}".format(i_fn+1, N))

        image = _load_image(fn, roi, e)
        
        if angle[0] > 0:
            color = (255, 0, 0)
        else:
            color = (0,0,255)

        image = _box(image, roi, lw, color=color)
        
        image = _crop(image, crop_roi, crop_factor)

        savefn = os.path.join(savedir, 'image_{:08d}.png'.format(i_frame))
        tifffile.imsave(savefn, image.astype(np.uint8))
        i_frame += 1

        if _draw_arrow:
            vx, vy = (roi[0] - crop_roi[0], roi[1] - crop_roi[1])
            vx, vy = np.array([vx, -vy]) / normalisations[os.path.basename(os.path.dirname(fn))]
            draw_arrow_inset(os.path.join(savedir, 'inset', 'image_{:08d}.png'.format(i_frame)), vx, vy,
                    color='white')
            

        if rel_rotation_time and angle != angles[i_fn+1]:
            next_image = _load_image(image_fns[i_fn+1], roi, e)
            if angles[i_fn+1][0] > 0:
                color = (255, 0, 0)
            else:
                color = (0,0,255)
            next_image = _box(next_image, ROIs[i_fn+1], lw, color=color)
            
            for blend in np.zeros(5).tolist() + np.linspace(0, 1, 25).tolist():
                im = image*(1-blend) + _crop(next_image, ROIs[i_fn+1], crop_factor)*(blend)
                savefn = os.path.join(savedir, 'image_{:08d}.png'.format(i_frame))
                tifffile.imsave(savefn, im.astype(np.uint8))
                
                if _draw_arrow:
                    draw_arrow_inset(os.path.join(savedir, 'inset', 'image_{:08d}.png'.format(i_frame)), vx, vy,
                            color='white')

                i_frame += 1


def _get_closest(folder_a, folders_b, index=False):
    '''
    pos

    returns closets_folder, distance
    '''

    dists = []

    A = np.array( folder_a )
    for b in folders_b:
        B = np.array( b )
        dists.append( np.linalg.norm(A-B) )
    
    if index:
        return np.argmin(dists), np.min(dists)
    else:
        return folders_b[np.argmin(dists)], np.min(dists)



def moving_rois_mosaic(manalysers, common_threshold=7.5, **kwargs):
    '''
    Uses moving_rois() to make a mosaic video of the experiments, in which
    the specimens move in sync.

    The first specimen (manalyser[0]) determines the rotation order (in the
    order as it was recorded).


    ARGUMENTS
    ---------
    common_threshold : int
        In rotation stage steps, how close the recordings of different
        analysers have to be classified as the same.
    kwargs : dict
        Passed to moving_rois
    
    RETURNS
    -------
    None
    '''

    orders = {manalyser.name: [] for manalyser in manalysers}
    excludes = {}

    folders = [angles_from_fn(os.path.basename(os.path.dirname(fn))) for fn in manalysers[0].get_time_ordered(first_frame_only=True)[0]]

    has_matches = {fol: 0 for fol in folders}
    conversion = {manalyser.name: {} for manalyser in manalysers}

    for folder in folders:
        for manalyser in manalysers[1:]:
            fols = [angles_from_fn(fol) for fol in manalyser.list_imagefolders(only_measured=True)] 
            closest, distance = _get_closest(folder, fols)
            
            if distance < common_threshold:
                orders[manalyser.name].append(closest)
                has_matches[folder] += 1
                conversion[manalyser.name][closest] = folder
    
    orders[manalysers[0].name] = folders.copy()
    orders[manalysers[0].name] = [fol for fol in orders[manalysers[0].name] if has_matches[fol] == len(manalysers)-1]   
    fols = [angles_from_fn(fol) for fol in manalysers[0].list_imagefolders(only_measured=True)]
    excludes[manalysers[0].name] = [fol for fol in fols if not fol in orders[manalysers[0].name]]
    
    for manalyser in manalysers[1:]:
        orders[manalyser.name] = [fol for fol in orders[manalyser.name] if has_matches[conversion[manalyser.name][fol]] == len(manalysers)-1]
       
        fols = [angles_from_fn(fol) for fol in manalyser.list_imagefolders(only_measured=True)]
        excludes[manalyser.name] = [fol for fol in fols if not fol in orders[manalyser.name]]
 
    
    # FIXME Starts too many processes with many specimens, possibly leading to
    # out of RAM

    processes = []
    
    for manalyser in manalysers:
        p = mp.Process(target=moving_rois, args=[manalyser],
                kwargs={ **{"_exclude_imagefolders": excludes[manalyser.name],
                    "_order": orders[manalyser.name],
                    "_draw_arrow": True}, **kwargs})
        p.start()
        
        processes.append(p)
        
    for p in processes:
        p.join()



def illustrate_experiments(manalyser, rel_rotation_time=1):
    '''
    Create a visualizing video how the vectormap is built.

    Arguments
    ---------
    rel_rotation_time : int or float
        Relative time spend on incrimentally rotating the vectormap
        between the stimuli.
    '''
    print('illustrate_experiments')
    
    savedir = os.path.join(ANALYSES_SAVEDIR, 'illustrate_experiments', manalyser.get_specimen_name())
    os.makedirs(savedir, exist_ok=True)
 
    fig = plt.figure(figsize=(6,6))
    ax = fig.add_subplot(1,1,1, projection='3d')
    plt.axis('off') 
    plt.subplots_adjust(left=-0.2, bottom=-0.2, right=1.2, top=1.2, wspace=0.0, hspace=0)


    # Points, vectors, angles
    lp, lv, la = manalyser.get_3d_vectors('left', return_angles=True)
    rp, rv, ra = manalyser.get_3d_vectors('right', return_angles=True)    
    
    image_fns, ROIs, angles = manalyser.get_time_ordered()
    arrow_artists = []

    lpoints, lvectors = [[],[]]
    rpoints, rvectors = [[],[]]

    print(len(image_fns))

    print(len(ROIs))
    print(len(angles))

    i_frame = 0
    for i_angle, (image_fn, ROI, angle) in enumerate(zip(image_fns, ROIs, angles)): 
        
        length_scale = (i_angle%20)/20 + 0.5
        final_image = (i_angle+1)%20 == 0

        #if not final_image:
        #   continue
        # Fix to account mirror_horizontal in 2D vectors
        angle = [-1*angle[0], angle[1]]


        # Calculate cameras place
        #x,y,z = coordinates.camera2Fly(*angle)
        #r, phi, theta = coordinates.to_spherical(x, y, z, return_degrees=True)

        #elev = float(90-theta)
        azim = -angle[0]+90
        
        if angle in la:
            indx = la.index(angle)
            
            nlp = coordinates.rotate_about_x(lp[indx], -angle[1])
            nlv = coordinates.rotate_about_x(lp[indx]+lv[indx], -angle[1]) - nlp

            if final_image:
                lpoints.append(lp[indx])
                lvectors.append(lv[indx])
            else:
                arrow_artists.extend(vector_plot(ax, [nlp], [length_scale*nlv], color='red'))

        
        if angle in ra:
            indx = ra.index(angle)

            nrp = coordinates.rotate_about_x(rp[indx], -angle[1])
            nrv = coordinates.rotate_about_x(rp[indx] + rv[indx], -angle[1]) - nrp

            if final_image:
                rpoints.append(rp[indx])
                rvectors.append(rv[indx])
            else:
                arrow_artists.extend(vector_plot(ax, [nrp], [length_scale*nrv], color='blue'))
        
        if lpoints:
            tmp_lpoints, tmp_lvectors = coordinates.rotate_vectors(np.array(lpoints), np.array(lvectors), 0, -math.radians(angle[1]), 0)
        
            arrow_artists.extend(vector_plot(ax, tmp_lpoints, tmp_lvectors, color='red', mutation_scale=3,
                    camerapos=[0,azim]))
            
        
        if rpoints:
            tmp_rpoints, tmp_rvectors = coordinates.rotate_vectors(np.array(rpoints), np.array(rvectors), 0, -math.radians(angle[1]), 0)
        

            arrow_artists.extend(vector_plot(ax, tmp_rpoints, tmp_rvectors, color='blue', mutation_scale=3,
                    camerapos=[0,azim]))

            
        #ax.dist = 2
        ax.view_init(elev=0, azim=azim)
        print('Horizontal {}, vertical {}'.format(*angle))

        savefn = os.path.join(savedir, 'image_{:08d}.png'.format(i_frame))
        fig.savefig(savefn, dpi=300, transparent=True)
        i_frame += 1
        
        # Rotating the saved image
        #camera_rotation = coordinates.correct_camera_rotation(*angle, return_degrees=True)
        #saved_image = Image.open(savefn)
        #saved_image.rotate(-camera_rotation).save(savefn)

        # Final image of this location, rotate the plot to the
        # new location
        if final_image and rel_rotation_time:
            next_angle = angles[i_angle+1]
            duration = 25
            hold_duration = 5
            
            for i, (h, v) in enumerate(zip(np.linspace(-angle[0], next_angle[0], duration), np.linspace(angle[1], next_angle[1], duration))):
                
                angle = [-h,v]
                azim = -angle[0]+90
                
                # FIXME The following part is copy paste from the part above
                # -> put it behind one function etc.

                # Clear arrows
                for arrow_artist in arrow_artists:
                    arrow_artist.remove()
                arrow_artists = []
                 
                # Redraw arros
                if lpoints:
                    tmp_lpoints, tmp_lvectors = coordinates.rotate_vectors(np.array(lpoints), np.array(lvectors), 0, -math.radians(angle[1]), 0)
                
                    arrow_artists.extend(vector_plot(ax, tmp_lpoints, tmp_lvectors, color='red', mutation_scale=3,
                            camerapos=[0,azim]))
                    
                
                if rpoints:
                    tmp_rpoints, tmp_rvectors = coordinates.rotate_vectors(np.array(rpoints), np.array(rvectors), 0, -math.radians(angle[1]), 0)
                

                    arrow_artists.extend(vector_plot(ax, tmp_rpoints, tmp_rvectors, color='blue', mutation_scale=3,
                            camerapos=[0,azim]))

                
                ax.view_init(elev=0, azim=azim)
                
                savefn = os.path.join(savedir, 'image_{:08d}.png'.format(i_frame))
                fig.savefig(savefn, dpi=300, transparent=True)
                i_frame += 1
  
                if i == 0:
                    for repeat in range(hold_duration):
                        savefn = os.path.join(savedir, 'image_{:08d}.png'.format(i_frame))
                        fig.savefig(savefn, dpi=300, transparent=True)
                        i_frame += 1
     


        for arrow_artist in arrow_artists:
            arrow_artist.remove()
        arrow_artists = []
    
    plt.close()



def rotation_mosaic(manalyser, imsize=(512,512),
        e=50, crop_factor=0.5):
    '''
    A mosaic (matrix) of the taken images.
    
    Arguments
    ---------
    manalyser : obj
        Analyser object
    n_vecticals : int
        How many vertical rotations rows to show
    n_horizontals : int
        How many horizontal rotation columns to show
    e, crop_factor
    '''

    # Part 1) Find rotations matching the interpolation

    rotations = manalyser.list_rotations()
    
    kdtree = KDTree(rotations)

    hrots, vrots = zip(*rotations)

    hmin, hmax = (np.min(hrots), np.max(hrots))
    vmin, vmax = (np.min(vrots), np.max(vrots))
   
    hstep = int(10 * (1024/360))
    vstep = int(10 * (1024/360))
    
    plot_data = []

    intp_v = np.arange(vmin, vmax, vstep)[::-1]
    intp_h = np.arange(hmin, hmax, hstep)

    for i_v, vrot in enumerate(intp_v):
        for i_h, hrot in enumerate(intp_h):
            
            if used_scipy_version < (1,6,0):
                distance, i_point = kdtree.query( (hrot, vrot), n_jobs=-1)
            else:
                distance, i_point = kdtree.query( (hrot, vrot), workers=-1)
            
            if distance > math.sqrt((hstep/1.5)**2 + (vstep/1.5)**2):
                continue

            plot_data.append((i_v, i_h, i_point))
            

    # Part 2: Plot the images

    image_fns, ROIs, angles = manalyser.get_time_ordered(angles_in_degrees=False,
            first_frame_only=True)
    
    w_mosaic = int(imsize[0]*len(intp_h))
    h_mosaic = int(imsize[1]*len(intp_v))
    mosaic = 255 * np.ones( (h_mosaic, w_mosaic) )
    
    print('Mosaic shape {}'.format(mosaic.shape))
    
    for i_plot_data, (i_v, i_h, i_point) in enumerate(plot_data): 
        print("{}/{}".format(i_plot_data+1, len(plot_data)))

        x = i_v * imsize[0]
        y = i_h * imsize[1]

        try:
            index = angles.index(list(rotations[i_point]))
        except ValueError:
            continue
        
        try:
            image = _load_image(image_fns[index], ROIs[index], 50) 
        except:
            continue

        image = image[0, :, :]
        image = cv2.resize(image, dsize=(*imsize,))        

        mosaic[x:x+imsize[0], y:y+imsize[1]] = image
    
    drot = manalyser.get_rotstep_size()

    fig, ax = plt.subplots(figsize=(10,10))
    ax.imshow(mosaic, cmap='gray', extent=[hmin*drot, hmax*drot, vmin*drot, vmax*drot])
    
    fig.savefig(os.path.join(ANALYSES_SAVEDIR, 'illustrate_experiments', "mosaic_"+manalyser.name+'.jpg'),
            dpi=600)

    plt.close()



'''
Attribute ANALYSER_CMDS dict here contains functions that accept
the Analyser object as their only argument.
'''

import numpy as np

from gonioanalysis.drosom import plotting
from gonioanalysis.drosom.plotting.common import save_3d_animation
from gonioanalysis.drosom.plotting import basics, illustrate_experiments, start_position
from gonioanalysis.drosom.plotting import complete_flow_analysis, error_at_flight
from gonioanalysis.drosom.special.norpa_rescues import norpa_rescue_manyrepeats
from gonioanalysis.drosom.special.paired import cli_group_and_compare
import gonioanalysis.drosom.reports as reports
import gonioanalysis.drosom.export as export

I_WORKER = None
N_WORKERS = None


# Functions that take only one input argument that is the MAnalyser
ANALYSER_CMDS = {}
ANALYSER_CMDS['pass'] = print
ANALYSER_CMDS['vectormap'] = basics.plot_3d_vectormap
ANALYSER_CMDS['vectormap_video'] = lambda analyser: save_3d_animation(analyser, plot_function=basics.plot_3d_vectormap, guidance=True, i_worker=I_WORKER, N_workers=N_WORKERS) 
ANALYSER_CMDS['export_vectormap'] = export.export_vectormap
ANALYSER_CMDS['magtrace'] = basics.plot_1d_magnitude
ANALYSER_CMDS['2d_vectormap'] =  basics.plot_2d_vectormap
ANALYSER_CMDS['trajectories'] = basics.plot_xy_trajectory

# Analyser + image_folder
ANALYSER_CMDS['moving_rois_video'] = illustrate_experiments.moving_rois
ANALYSER_CMDS['illustrate_experiments_video'] = illustrate_experiments.illustrate_experiments
ANALYSER_CMDS['rotation_mosaic'] = illustrate_experiments.rotation_mosaic
ANALYSER_CMDS['norpa_rescue_manyrepeats'] = norpa_rescue_manyrepeats
ANALYSER_CMDS['compare_paired'] = cli_group_and_compare
ANALYSER_CMDS['lr_displacements'] = lambda analyser: reports.left_right_displacements(analyser, 'test')
ANALYSER_CMDS['left_right_summary'] = reports.left_right_summary
ANALYSER_CMDS['pdf_summary'] = reports.pdf_summary

rotations = np.linspace(-180,180, 360)
ANALYSER_CMDS['flow_analysis_yaw'] = lambda analyser: complete_flow_analysis(analyser, rotations, 'yaw')
ANALYSER_CMDS['flow_analysis_roll'] = lambda analyser: complete_flow_analysis(analyser, rotations, 'roll')
ANALYSER_CMDS['flow_analysis_pitch'] = lambda analyser: complete_flow_analysis(analyser, rotations, 'pitch')

ANALYSER_CMDS['error_at_flight'] = error_at_flight
ANALYSER_CMDS['startpos'] = start_position.start_heatmap

# Functions that take two input arguments;
# MAanalyser object and the name of the imagefolder, in this order
IMAGEFOLDER_CMDS = {}
IMAGEFOLDER_CMDS['magtrace'] = basics.plot_1d_magnitude


# Functions that take two manalyser as input arguments
DUALANALYSER_CMDS = {}
DUALANALYSER_CMDS['difference'] = basics.plot_3d_differencemap
DUALANALYSER_CMDS['compare'] = basics.compare_3d_vectormaps
DUALANALYSER_CMDS['compare_compact'] = basics.compare_3d_vectormaps_compact
DUALANALYSER_CMDS['compare_manyviews'] = basics.compare_3d_vectormaps_manyviews

DUALANALYSER_CMDS['difference_video'] = lambda analyser1, analyser2: save_3d_animation([analyser1, analyser2],
        plot_function=basics.plot_3d_differencemap, guidance=False, hide_axes=True, colorbar=False, hide_text=True,
        i_worker=I_WORKER, N_workers=N_WORKERS) 
DUALANALYSER_CMDS['export_differencemap'] = export.export_differencemap


# Manyviews videos
for animation_type in ['rotate_plot', 'rotate_arrows', 'pitch_rot', 'yaw_rot', 'roll_rot']:
    DUALANALYSER_CMDS['compare_manyviews_{}_video'.format(animation_type.replace('_',''))] = lambda an1, an2, at=animation_type: save_3d_animation([an1, an2], plot_function=basics.compare_3d_vectormaps_manyviews, animation_type=at)


# Functions that take in a list of manalysers (first positional argument)
MULTIANALYSER_CMDS = {}
MULTIANALYSER_CMDS['magnitude_probability'] = basics.plot_magnitude_probability
MULTIANALYSER_CMDS['moving_rois_mosaic'] = illustrate_experiments.moving_rois_mosaic






import os
import sys

import numpy as np

import matplotlib.pyplot as plt
import matplotlib.cm
import matplotlib.widgets
import tifffile

from tk_steroids.matplotlib import CanvasPlotter

from movemeter.stacks import stackread

from gonioanalysis.drosom.plotting.basics import (
        plot_1d_magnitude,
        plot_xy_trajectory,
        plot_3d_vectormap,
        plot_3d_magnitudemap,
        )

class RecordingPlotter:
    '''For plotting a single image folder and its data.

    Uses tk_steroids.matplotlib.CanvasPlotter to plot on matplotlib figures.
    
    Attributes
    -----------
    core
        Reference to the Core instance
    selected_recording
        The recording currently plotted at the plotter
    i_repeat
        None if show all traces, otherwise the index of repeat to be shown
    N_repeats
        The total number of repeats in the image folder.

    '''

    def __init__(self, core):
        '''
        core    An instance of the Core class in core.py
        '''

        self.core = core
        
        # Keeps internally track of the current recording on plot
        self.selected_recording = None

        self.colorbar = None
        self.roi_rectangles = []
        
        self.N_repeats = 0
        self.i_repeat = None

        self.restore_rotation = None

    def _check_recording(self, skip_datafetch=False):
        '''
        Check from core if the selected has changed.
        '''
        selected_recording = self.core.selected_recording

        if self.selected_recording != selected_recording:
            self.i_repeat = None
        
        self.selected_recording = selected_recording

        if not skip_datafetch:
            if self.core.analyser.is_measured():
                self.movement_data = self.core.analyser.get_movements_from_folder(selected_recording)
                self.N_repeats = len(next(iter(self.movement_data.values())))
                pass
            else:
                self.movement_data = {}
                self.N_repeats = 0
        


    def magnitude(self, ax, **kwargs):
        '''
        Plot a displacement over time of the current specimen/recording.
        '''
        self._check_recording(skip_datafetch=True)
        
        ax, self.magnitudes, self.N_repeats = plot_1d_magnitude(self.core.analyser,
                self.selected_recording,
                i_repeat=self.i_repeat,
                label='EYE-repIREPEAT',
                ax=ax,
                **kwargs)


    def vectormap(self, ax, **kwargs):

        self._check_recording()
        
        self.N_repeats = 0
       
        if self.restore_rotation:
            elev = None
            azim = None
        else:
            elev = ax.elev
            azim = ax.azim

        ax, self.vectors = plot_3d_vectormap(
                self.core.analyser,
                image_folder=self.selected_recording,
                elev=elev, azim=azim,
                ax=ax,
                **kwargs)

    def magnitudemap(self, ax, **kwargs):
        
        self.N_repeats = 0
       
        if self.restore_rotation:
            elev = None
            azim = None
        else:
            elev = ax.elev
            azim = ax.azim

        ax, self.vectors = plot_3d_magnitudemap(
                self.core.analyser, ax=ax,
                elev=elev, azim=azim,
                **kwargs)

    def xy(self, ax, **kwargs):
        '''
        Plot (x, y) where time is encoded by color.
        '''
        self._check_recording()
        
        ax, self.xys = plot_xy_trajectory([self.core.analyser],
                {self.core.analyser.name: [self.selected_recording]},
                i_repeat=self.i_repeat,
                ax=ax,
                **kwargs)

    def imshow(self, sequence_imshow):
        '''Show images on a slider

        Arguments
        ---------
        sequence_imshow : SequenceImshow
            From tksteroids matplotlib
        '''
        self._check_recording(skip_datafetch=True)

        self.N_repeats = self.core.analyser.get_N_repeats(
                self.selected_recording)
        
        if self.i_repeat:
            i_repeat = self.i_repeat
        else:
            i_repeat = 0
 
        image_fns = self.core.analyser.list_images(
                self.selected_recording, i_repeat=i_repeat,
                absolute_path=True)
        
        # FIXME: Reading all in memory wont work with huge data
        if len(image_fns) == 1:
            # Stack
            images = [im[::2,::2] for im in stackread(image_fns[0])]
        else:
            # Separate images
            images = [stackread(fn)[0][::2,::2] for fn in image_fns]
        sequence_imshow.imshow(images, cmap='gray', slider=True)


    def ROI(self, canvas_plotter):
        '''
        Plot specimen/recording image, and the ROIs and imaging parameters on top of it.
        '''
        self._check_recording(skip_datafetch=True)
        
        fig, ax = canvas_plotter.get_figax()
        
        # Plot the image
        image_fns = self.core.analyser.list_images(self.selected_recording, absolute_path=True)
        self.N_repeats = len(image_fns)

        if self.i_repeat:
            i_frame = self.i_repeat
        else:
            i_frame = 0
        
        image_fn = image_fns[i_frame]
        self.image = tifffile.TiffFile(image_fn).asarray(key=0)

        canvas_plotter.imshow(self.image, cmap='gray', slider=True)


        # Draw ROI rectangles
        for old_roi in self.roi_rectangles:
            try:
                old_roi.remove()
            except NotImplementedError:
                # This error occurs when something goes from from here on before
                # the ax.add_patch(roi) line. We can just ignore this as the
                # ROI has not been ever added to the ax
                continue
        self.roi_rectangles = []
        
        for roi in self.core.analyser.get_rois(self.selected_recording):
            patch = matplotlib.patches.Rectangle((roi[0], roi[1]), roi[2], roi[3],
                    fill=False, edgecolor='White')
            self.roi_rectangles.append(patch)
        
        for roi in self.roi_rectangles:
            ax.add_patch(roi)
        

        # Text about imaging parameters
        imaging_params = self.core.analyser.get_imaging_parameters(self.selected_recording)
        if imaging_params:
            text = '\n'.join(['{}: {}'.format(setting, value) for setting, value in imaging_params.items()])
        else:
            text = 'Unable to fetch imaging parameters'

        try:
            self.roi_text.set_text(text)
        except AttributeError:
            self.roi_text = ax.text(0,1, text, ha='left', va='top', fontsize='small', 
                    transform=ax.transAxes)


def main():
    pass


if __name__ == "__main__":
    main()





import matplotlib.pyplot as plt
import matplotlib.cm

from ..kinematics import sigmoidal_fit
from ..startpos_analysis import StartposAnalyser
from ..analysing import MAnalyser

def start_heatmap(spanalyser, image_folder=None, ax=None):
    '''Plots heatmap how the feature starting position affects the movement.

    spanalyser : obj
        Movement analyser
    '''
    if isinstance(spanalyser, StartposAnalyser):
        manalyser = MAnalyser(spanalyser.data_path, spanalyser.folder)
    elif isinstance(spanalyser, MAnalyser):
        manalyser = spanalyser
        spanalyser = StartposAnalyser(manalyser.data_path, manalyser.folder)
    else:
        raise ValueError(f'Spanalyser has to be a StartposAnalyser (or MAnalyser for flip)')

    if ax is None:
        fig, ax = plt.subplots()
    else:
        fig = ax.get_figure()
        
    if image_folder is None:
        image_folder = manalyser.list_imagefolders(only_measured=True)
    elif not isinstance(image_folder, list):
        image_folder = [image_folder]

    xys = []

    for folder in image_folder:
        amplitudes, speeds, latencies = sigmoidal_fit(
                manalyser, folder, fit_to_mean=False)

        movements = spanalyser.get_movements_from_folder(folder)
       
        for eye in movements:
            X = movements[eye][0]['x']
            Y = movements[eye][0]['y']
            print(X)
            print(Y)
            print(amplitudes)
            if len(X) != len(amplitudes):
                continue
            if len(Y) != len(amplitudes):
                continue

            ax.plot(X,Y, color='gray')
            sc = ax.scatter(X, Y, c=amplitudes)
            
            if not hasattr(ax, 'startcolorbar'):
                #ax.startcolorbar.remove()
                ax.startcolorbar = plt.colorbar(sc, ax=ax)
            
            xys.append([X,Y])
            break
 
    return xys



import os
import unittest

import gonioanalysis.drosom.analyser_commands as ac
import gonioanalysis.drosom.terminal as terminal

TESTPATH = os.path.dirname(__file__)

datadir = os.path.join(TESTPATH, 'test_data')
testspecimen = 'test_specimen_01'

class TestTerminal(unittest.TestCase):
    
    def test_analysis_targets(self):
        '''
        Test all the analysis targets in the terminal.py, ie essentially
    
        terminal.py -D datadir -S testspecimen analysis1
        terminal.py -D datadir -S testspecimen analysis2
        ...

        only checking if they can be run successfully without errors.
        '''

        targets = ac.ANALYSER_CMDS

        for target in targets:
            with self.subTest(target=target):
                args = ['--dont-show','-D', datadir, '-S', testspecimen, target]
                terminal.main(custom_args=args)



'''Interactive tkinter widget for calibrating the vertical zero-rotation.
'''

import os
import numpy as np

import tkinter as tk

from tk_steroids.matplotlib import CanvasPlotter
from movemeter.stacks import stackread

from gonioanalysis.antenna_level import (
        load_drosom,
        save_antenna_level_correction,
        load_reference_fly,
        )


class ZeroCorrect(tk.Frame):
    '''
    Creates a frame where one can perform zero correction (atenna level search)
    for a specimen using reference specimen (alr, antenna level reference)
    
    This is (maybe better) alternative to the command line tool, antenna_level.py,
    that uses binary search tactics to match.
    '''

    def __init__(self, tk_parent, specimen_path, alr_data_path, callback=None):
        tk.Frame.__init__(self, tk_parent)
        self.parent = tk_parent
        self.callback = callback

        self.specimen_name = os.path.basename(specimen_path)

        # Load data
        self.specimen_pitches, self.specimen_images = load_drosom(specimen_path)
        #self.reference_pitches, self.reference_images = {fn: pitch for pitch, fn in loadReferenceFly(alr_data_path).items()}
       
        try:
            alr_data = load_reference_fly(alr_data_path)
        except FileNotFoundError:
            alr_data = {}

        self.reference_pitches, self.reference_images = [[],[]]
        for pitch, fn in sorted(alr_data.items(), key=lambda x: float(x[0])):
            self.reference_pitches.append(pitch)
            self.reference_images.append(fn)

        # Set plotters
        self.specimen_plotter = CanvasPlotter(self, text=specimen_path)
        self.specimen_plotter.grid(row=1, column=0, sticky='NSWE')

        self.reference_plotter = CanvasPlotter(self, text='Reference fly')
        self.reference_plotter.grid(row=1, column=1, sticky='NSEW')
        
        if not alr_data:
            self.reference_plotter.ax.text(0.1, 0.1, (
                    "No reference data created. Options:\n"
                    "  A) Set correction manually\n"
                    "  B) Run scripts/create_alr_data.py"),
                    color='red')

        self.grid_columnconfigure(0, weight=1)
        self.grid_columnconfigure(1, weight=1)
        self.grid_rowconfigure(1, weight=1)
        

        # Help text
        tk.Label(self, text='Rotate the reference until it matches the specimen and press Next image.\nAlternatively, set manual correction.').grid(row=0, column=0, columnspan=2)

        # Set buttons
        buttons_frame = tk.LabelFrame(self, text='Rotate reference')
        buttons_frame.grid(row=2, column=0, columnspan=2)
        steps = [-20, -5, -3, -1, 1, 3, 5, 20]
        for i_column, step in enumerate(steps):
            button = tk.Button(buttons_frame, text=str(step), command=lambda step=step: self.rotate_reference(step))
            button.grid(row=1, column=i_column)
        
        self.set_button = tk.Button(self, text='Next image', command=self.set_image)
        self.set_button.grid(row=3, column=0, columnspan=2)
        
        self.set_manual_button = tk.Button(self, text='Set manual correction...', command=self.set_manual)
        self.set_manual_button.grid(row=3, column=1, sticky='E')
        

        # Loop variables
        self.i_specimen = 0
        self.i_reference = 0
    
        # Offset between each specimen-reference image is saved here.
        self.offsets = []
        
        self.update_plots() 


    def rotate_reference(self, steps):
        '''
        When user clicks to rotate the reference fly.
        '''
        self.i_reference += steps
        
        if self.i_reference >= len(self.reference_pitches):
            self.i_reference = len(self.reference_pitches) - 1
        elif self.i_reference < 0:
            self.i_reference = 0
        
        self.update_plots()
        

    def set_image(self):
        '''
        When user sets the current reference rotation as the best match
        '''
        offset = float(self.specimen_pitches[self.i_specimen]) - float(self.reference_pitches[self.i_reference])
        self.offsets.append(offset)
        self.i_specimen += 1

        if self.i_specimen == len(self.specimen_pitches):
            self.report()
        else:
            self.update_plots()
   

    def update_plots(self):
        '''
        Call to update imshow plots.
        '''
        if self.reference_images:
            self.reference_image = next(stackread(self.reference_images[self.i_reference]))
            self.reference_plotter.imshow(self.reference_image, cmap='gray', slider=True)

        self.specimen_image = next(stackread(self.specimen_images[self.i_specimen]))
        self.specimen_plotter.imshow(self.specimen_image, cmap='gray', slider=True)

    
    def set_manual(self):
        '''
        Let the user specify a manual correction, skipping the rotation process.
        '''
        value = tk.simpledialog.askstring("Manual correction value",
                "The vertical angle when the deep\npseudopupils align with the antenna?", parent=self)
        
        if value:
            self.offsets = float(value)
            self.report()


    def report(self):
        '''
        Report the results with a pop up window
        '''
        message = 'Correction value set as {}'.format(np.mean(self.offsets))
        tk.messagebox.showinfo('Zero correction ready', message, parent=self)
        
        save_antenna_level_correction(self.specimen_name, np.mean(self.offsets))
        
        if self.callback:
            self.callback()

        self.destroy()




'''
A script to autogenerate .cmd files for Windows in the bin.
'''

import os

def main():

    os.chdir('bin')
    
    pyfiles = [fn for fn in os.listdir() if fn.endswith('.py')]

    for pyfile in pyfiles:
        # Here assuming there's only one . in the filenames
        cmd_fn = pyfile.split('.')[0] + '.cmd'
        with open(cmd_fn, 'w') as fp:
            fp.write('python {}'.format(pyfile))

if __name__ == "__main__":
    main()



'''
Linked Biosyst electrophysiology data, intended for electroretinograms (ERGs).
'''

import os
import csv
import glob

import numpy as np

from gonioanalysis.directories import ANALYSES_SAVEDIR

from biosystfiles import extract as bsextract

  
def _load_ergs(ergs_labbook, ergs_rootdir):
    '''
    Fetches ERGs for the specimen matching the name.

    Requirements
    - ERGs are Biosyst recorded .mat files.
    - Requires also a lab book that links each specimen name to a ERG file,
        and possible other parameter values such as intensity, repeats,
        UV/green etc.

    Returns ergs {specimen_name: data}
        where data is a list [[ergs, par1, par2, ...],[..],..]
        ergs are np arrays
    '''
    ergs = {}

    csvfile = []
    with open(ergs_labbook, 'r') as fp:
        reader = csv.reader(fp)
        for row in reader:
            csvfile.append(row)
    
    previous_specimen = ''

    # Header (column names)
    column_names = csvfile[0]

    for line in csvfile[1:]:
        efn = line[1]
        match = glob.glob(ergs_rootdir+'/**/'+efn)
        if len(match) != 1:
            print('{} not found'.format(efn))
        else:
            specimen = line[0]
            if not specimen:
                specimen = previous_specimen
            previous_specimen = specimen

            try:
                ergs[specimen]
            except KeyError:
                ergs[specimen] = []

            ddict = {key: value for key, value in zip(column_names[2:], line[2:])}
            trace, fs = bsextract(match[0], 0)

            # Use the mean if many repeats present
            ddict['data'] = np.mean(trace, axis=1).flatten().tolist()
            ddict['fs'] = int(fs)
            ergs[specimen].append(ddict)

    return ergs



def link_erg_labbook(manalysers, ergs_labbook, ergs_rootdir):
    '''
    Links MAnalyser objects with erg data fetched from an electronic
    labbook. For specification, see function _load_ergs
    '''

    erg_data = _load_ergs(ergs_labbook, ergs_rootdir)

    for manalyser in manalysers:
        mname = manalyser.get_specimen_name()
        
        if mname in erg_data:
            manalyser.link_data('ERGs', erg_data[mname])
            manalyser.save_linked_data()

        else:
            print('No ERGs for {}'.format(mname))





'''

TODO

Features
+ video of the moving ROIs
+ no need for manual add to PythonPath on Windows
- window to strecth to full screen
- multiselect ROIs (if many separate recordings at the same site)

Polishing
- specimens control title, retain name specimen
- image folders control title as specimens
- after movement measure, update MAnalyser
- displacement plot Y axis label
- highlight specimens/image_folders:
    red: no rois / movements
    yellow: rois but no movements
- x-axis from frames to time?


'''
import os
import sys
import ctypes
import itertools

import numpy as np
import tkinter as tk
from tkinter import filedialog

from tk_steroids.routines import inspect_booleans
from tk_steroids.elements import (
        Listbox,
        Tabs,
        ButtonsFrame,
        ColorExplanation,
        TickboxFrame
        )
from tk_steroids.matplotlib import CanvasPlotter, SequenceImshow

from gonioanalysis import __version__
from gonioanalysis.directories import PROCESSING_TEMPDIR, GONIODIR
from gonioanalysis.rotary_encoders import to_degrees
from gonioanalysis.drosom.loading import angles_from_fn
from gonioanalysis.drosom.plotting.common import save_3d_animation
from gonioanalysis.drosom.plotting.basics import (
        plot_1d_magnitude, 
        plot_xy_trajectory,
        plot_3d_vectormap,
        )
from gonioanalysis.drosom.plotting.ax3d import Cp3d
from gonioanalysis.drosom.analyser_commands import ANALYSER_CMDS
from gonioanalysis.tkgui.core import Core
from gonioanalysis.tkgui.plotting import RecordingPlotter
from gonioanalysis.tkgui.widgets import (
        RepetitionSelector,
        SetPlotlimits,
        )

from gonioanalysis.tkgui.menu_commands import (
        ModifiedMenuMaker,
        FileCommands,
        ImageFolderCommands,
        SpecimenCommands,
        ManySpecimenCommands,
        OtherCommands
        )



class ExamineMenubar(tk.Frame):
    '''
    Menubar class for the examine GUI.
    '''

    def __init__(self, parent):
        tk.Frame.__init__(self, parent)

        self.parent = parent
        self.root = parent.root
        self.core = parent.core
        self.menubar = tk.Menu(self)

        # File command and menu
        self.file_commands = FileCommands(self.parent, self.core, 'File')
        self.file_commands._connect(self.menubar, tearoff=0)
        
        self.menubar.add_command(label="|")

        # Imagefolder command and menu
        self.imagefolder_commands = ImageFolderCommands(self.parent, self.core, 'Image folder')
        self.imagefolder_commands._connect(self.menubar, tearoff=0)

        # Specimen commands and menu
        self.specimen_commands = SpecimenCommands(self.parent, self.core, 'Specimen')
        self.specimen_commands._connect(self.menubar, tearoff=0)
        
        #    Submenu: Add terminal commands
        self.terminal_commands = ModifiedMenuMaker(self.parent, self.core, 'Terminal interface commands')
        for name in ANALYSER_CMDS:
            setattr(self.terminal_commands, name, lambda name=name: self.core.adm_subprocess('current', "-A "+name) )
        self.terminal_commands._connect(self.specimen_commands.tkmenu)

        # Many specimen commands and menu
        self.many_specimen_commands = ManySpecimenCommands(self.parent, self.core, 'Many specimens')
        self.many_specimen_commands._connect(self.menubar, tearoff=0)
       
        self.menubar.add_command(label="|")
        
        # Other commands and menu
        self.other_commands = OtherCommands(self.parent, self.core, 'Other')
        self.other_commands._connect(self.menubar, tearoff=0)
        

        self.winfo_toplevel().config(menu=self.menubar)
   

class VectormapControls(tk.Frame):
    '''Controls for the vectormap plots
    '''

    def __init__(self, parent, command=None):
        super().__init__(parent)
        
        # Controls for the vector plot
        # Controls for displacement plot (means etc)
        vectorplot_options, vectorplot_defaults = inspect_booleans(
                plot_3d_vectormap, exclude_keywords=[])

        self.ticks = TickboxFrame(
                self, vectorplot_options,
                defaults=vectorplot_defaults,
                callback=command,
                ncols=4)
        self.ticks.grid(row=0, column=0, columnspan=2)

        self.merge_slider = tk.Scale(
                self, from_=0, to=1, resolution=0.01,
                orient=tk.HORIZONTAL,
                command=command)
        self.merge_slider.grid(row=1, column=1, sticky='WE')
        tk.Label(self, text='Merge by distance').grid(
                row=1, column=0)
        
        self.columnconfigure(1, weight=1)


    @property
    def merge_distance(self):
        return float(self.merge_slider.get())

    @property
    def settings(self):
        return {**self.ticks.states,
                'merge_distance': self.merge_distance}
        

class PlotView(tk.Frame):
    '''The plot showing part of the examine view (the old rightside_frame).
    '''
    def __init__(self, examine, core):

        tk.Frame.__init__(self, examine)
        self.examine = examine
        self.core = core

        tab_kwargs = [
                {'visibility_button': False},
                {'visibility_button': False},
                {'visibility_button': False},
                {'visibility_button': False},
                {'projection': '3d', 'visibility_button': False},
                {'projection': '3d', 'visibility_button': False},
                {}]
        tab_names = [
                'ROI', 'Images', 'Displacement', 'XY',
                '3D Dir', '3D Mag', '3D New']
        classes = [CanvasPlotter, SequenceImshow,
                   CanvasPlotter, CanvasPlotter,
                   CanvasPlotter, CanvasPlotter,
                   Cp3d]
        canvas_constructors = []
        for clas, kwargs in zip(classes, tab_kwargs):
            con = lambda parent, cl=clas, kwargs=kwargs: cl(parent, **kwargs)
            canvas_constructors.append(con)

        self.tabs = Tabs(self, tab_names, canvas_constructors,
                on_select_callback=self.update_plot)
        
        self.tabs.grid(row=0, column=0, sticky='NWES')


        # Make canvas plotter to stretch
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)


        self.canvases = self.tabs.get_elements()
        
        # Controls for displacement plot (means etc)
        self.displacement_limits = SetPlotlimits(
                self.canvases[2], self.canvases[2].ax)
        self.displacement_limits.grid()

        displacementplot_options, displacementplot_defaults = inspect_booleans(
                plot_1d_magnitude, exclude_keywords=['mean_imagefolders'])
        self.displacement_ticks = TickboxFrame(self.canvases[2], displacementplot_options,
                defaults=displacementplot_defaults, callback=self.update_plot)
        self.displacement_ticks.grid()

        # XY Plot
        xyplot_options, xyplot_defaults = inspect_booleans(
                plot_xy_trajectory)
        self.xy_ticks = TickboxFrame(self.canvases[3], xyplot_options,
                defaults=xyplot_defaults, callback=self.update_plot)
        self.xy_ticks.grid()


        # Vectormap options
        self.vplot1_opts = VectormapControls(
                self.canvases[4], command=lambda x: self.update_plot())
        self.vplot1_opts.grid(sticky='WE')
        self.vplot2_opts = VectormapControls(
                self.canvases[6].tk, command=lambda x: self.update_plot())
        self.vplot2_opts.grid(sticky='WE')
        
        # Hide vplot2.ticks for now; Does not properly make sense yet
        self.vplot2_opts.ticks.grid_remove()


        self.canvases[4].get_figax()[1].elev = 10 
        self.canvases[4].get_figax()[1].azim = 70
        self.canvases[6].get_figax()[1].elev = 10
        self.canvases[6].get_figax()[1].azim = 70

        tk.Button(self.canvases[4], text='Save animation', command=self.save_3d_animation).grid()


        self.plotter = RecordingPlotter(self.core)
                
        # Add buttons for selecting single repeats from a recording
        self.repetition_selector = RepetitionSelector(self, self.plotter, self.core,
                update_command=lambda: self.examine.on_recording_selection('current'))
        self.repetition_selector.grid(row=1, column=0)
        

        tk.Button(self.repetition_selector, text='Copy data',
                command=self.copy_to_clipboard).grid(row=0, column=5)

        tk.Button(self.repetition_selector, text='Save view...',
                command=self.save_view).grid(row=0, column=6)
        
        self.columnconfigure(0, weight=1)

   
    def update_plot(self, i_plot=None):
        '''
        i_plot : int or None
            Index of the plot (from 0 to N-1 tabs) or None just to update the current open tab.
        '''
        if self.core.selected_recording is None:
            return None
        
        if i_plot is None:
            i_plot = self.tabs.i_current
        fig, ax = self.canvases[i_plot].get_figax()

        if i_plot == 0:
            self.plotter.ROI(self.canvases[i_plot])
        elif i_plot == 1:
            self.plotter.imshow(self.canvases[i_plot])
        else:
            
            ax.clear()

            remember_analysis = self.core.analyser.active_analysis

            for analysis in [name for name, state in self.examine.tickbox_analyses.states.items() if state == True]:
                
                self.core.analyser.active_analysis = analysis
                
                if i_plot == 2:
                    self.plotter.magnitude(ax, **self.displacement_ticks.states)
                    self.displacement_limits.apply()
                elif i_plot == 3:  
                    self.plotter.xy(ax, **self.xy_ticks.states) 
                elif i_plot == 4:
                    self.plotter.vectormap(
                            ax, **self.vplot1_opts.settings)
                elif i_plot == 5:
                    self.plotter.magnitudemap(ax)
                elif i_plot == 6:
                    self.plotter.vectormap(
                            ax, **self.vplot2_opts.settings)
                
            self.core.active_analysis = remember_analysis


        self.canvases[i_plot].update()
        
        self.repetition_selector.update_text()


    def copy_to_clipboard(self, force_i_tab=None):
        '''
        Copies data currently visible on theopen plotter tab to the clipboard.
        
        force_i_tab         Copy from the specified tab index, instead of
                            the currently opened tab
        '''
        formatted = ''
        
        # Always first clear clipboard; If something goes wrong, the user
        # doesn't want to keep pasting old data thinking it's new.
        self.examine.root.clipboard_clear()
        
        if self.core.selected_recording is None:
            return None

        if force_i_tab is not None:
            i_tab = int(force_i_tab)
        else:
            i_tab = self.tabs.i_current
        
        # Make sure we have the correct data in the plot by reissuing
        # the plotting command
        self.update_plot(i_tab)
        
        # Select data based on where we want to copy
        if i_tab == 0:
            data = self.plotter.image
        elif i_tab == 1:
            data = self.plotter.canvas_plotter.image
        elif i_tab == 2:
            data = self.plotter.magnitudes
        elif i_tab == 3:
            data = self.plotter.xys
            data = list(itertools.chain(*data))
        elif i_tab >= 4:
            raise NotImplementedError('Cannot yet cliboard 3D data')

        # Format the data for tkinter clipboard copy
        for i_frame in range(len(data[0])):
            formatted += '\t'.join([str(data[i_repeat][i_frame]) for i_repeat in range(len(data)) ]) + '\n'
        
        self.examine.root.clipboard_append(formatted)
        self.examine.copy_to_csv(formatted)


    def save_view(self):
        '''
        Launches a save dialog for the current plotter view.
        '''
        fig, ax = self.canvases[self.tabs.i_current].get_figax()
        
        dformats = fig.canvas.get_supported_filetypes()
        formats = [(value, '*.'+key) for key, value in sorted(dformats.items())]
        
        # Make png first
        if 'png' in dformats.keys():
            i = formats.index((dformats['png'], '*.png'))
            formats.insert(0, formats.pop(i))

        fn = filedialog.asksaveasfilename(title='Save current view',
                initialdir=self.examine.last_saveplotter_dir,
                filetypes=formats)

        if fn:
            self.last_saveplotter_dir = os.path.dirname(fn)

            fig.savefig(fn, dpi=1200)
        

    def save_3d_animation(self):
        
        def callback():
            self.canvases[3].update()
        
        fig, ax = self.canvases[3].get_figax()
        save_3d_animation(self.core.analyser, ax=ax, interframe_callback=callback)




class ExamineView(tk.Frame):
    '''
    The examine frame. Selection of
    - data directory
    - specimen
    - recording
    and plotting the intemediate result for each recording.
    
    '''
    
    def __init__(self, parent):

        self.last_saveplotter_dir = GONIODIR

        tk.Frame.__init__(self, parent)
        
        self.core = Core()
        self.core.update_gui = self.update_specimen
        
        self.root = self.winfo_toplevel()
        
        # Make canvas plotter to stretch
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(1, weight=3)
        self.grid_columnconfigure(0, weight=1)
        self.grid_columnconfigure(0, minsize=400)


        #tk.Button(self, text='Set data directory...', command=self.set_data_directory).grid(row=0, column=0)
        
        # Uncomment to have the menubar
        self.menu = ExamineMenubar(self)

        # LEFTSIDE frame
        self.leftside_frame = tk.Frame(self)
        self.leftside_frame.grid(row=0, column=0, sticky='NSWE') 
        self.leftside_frame.grid_rowconfigure(4, weight=1) 
        self.leftside_frame.grid_columnconfigure(0, weight=1)
        self.leftside_frame.grid_columnconfigure(1, weight=1)



        # The 1st buttons frame, selecting root data directory
        self.buttons_frame_1 = ButtonsFrame(self.leftside_frame, ['Set data directory'],
                [self.menu.file_commands.set_data_directory, self.menu.file_commands.set_data_directory])
        self.buttons_frame_1.grid(row=0, column=0, sticky='NW', columnspan=2)

 
        self.specimen_control_frame = tk.LabelFrame(self.leftside_frame, text='Specimen')
        self.specimen_control_frame.grid(row=1, column=0, sticky='NWES', columnspan=2)

       
        # The 2nd buttons frame, ROIs and movements
        self.buttons_frame_2 = ButtonsFrame(self.specimen_control_frame,
                ['Select ROIs', 'Measure movement'],
                [self.menu.specimen_commands.select_ROIs, self.menu.specimen_commands.measure_movement])
        self.buttons_frame_2.grid(row=1, column=0, sticky='NW', columnspan=2)
        self.button_rois, self.button_measure = self.buttons_frame_2.get_buttons()
        
        # Subframe for 2nd buttons frame
        #self.status_frame = tk.Frame(self.leftside_frame)
        #self.status_frame.grid(row=2)

        self.status_rois = tk.Label(self.specimen_control_frame, text='ROIs selected 0/0', font=('system', 8))
        self.status_rois.grid(row=2, column=0, sticky='W')
        
        self.status_antenna_level = tk.Label(self.specimen_control_frame, text='Zero correcter N/A', font=('system', 8))
        #self.status_antenna_level.grid(row=3, column=0, sticky='W')
        
        self.status_active_analysis = tk.Label(self.specimen_control_frame, text='Active analysis: default', font=('system', 8), justify=tk.LEFT)
        self.status_active_analysis.grid(row=4, column=0, sticky='W')
       
        self.tickbox_analyses = TickboxFrame(self.specimen_control_frame, [], ncols=4)
        self.tickbox_analyses.grid(row=5, column=0, sticky='W')


        # Image folder manipulations
        self.folder_control_frame = tk.LabelFrame(self.leftside_frame, text='Image folder')
        self.folder_control_frame.grid(row=2, column=0, sticky='NWES', columnspan=2)
       
        self.buttons_frame_3 = ButtonsFrame(self.folder_control_frame,
                ['Reselect ROI', 'Remeasure'],
                [self.menu.imagefolder_commands.select_ROIs, self.menu.imagefolder_commands.measure_movement])
        self.buttons_frame_3.grid(row=1, column=0, sticky='NW', columnspan=2)
        self.button_one_roi = self.buttons_frame_2.get_buttons()[0]
        

        self.status_horizontal = tk.Label(self.folder_control_frame, text='Horizontal angle N/A', font=('system', 8))
        self.status_horizontal.grid(row=2, column=0, sticky='W')
        
        self.status_vertical = tk.Label(self.folder_control_frame, text='Vertical angle N/A', font=('system', 8))
        self.status_vertical.grid(row=3, column=0, sticky='W')
        

        # Selecting the specimen 
        tk.Label(self.leftside_frame, text='Specimens').grid(row=3, column=0)
        self.specimen_box = Listbox(self.leftside_frame, ['(select directory)'], self.on_specimen_selection)
        self.specimen_box.grid(row=4, column=0, sticky='NSEW')

       
        # Selecting the recording
        tk.Label(self.leftside_frame, text='Image folders').grid(row=3, column=1)
        self.recording_box = Listbox(self.leftside_frame, [''], self.on_recording_selection)
        self.recording_box.grid(row=4, column=1, sticky='NSEW')

        
        # Add color explanation frame in the bottom
        ColorExplanation(self.leftside_frame, ['white', 'green', 'yellow'],
                ['Movements measured', 'ROIs selected', 'No ROIs']).grid(row=5, column=0, sticky='NW')


        self.default_button_bg = self.button_rois.cget('bg')
        
        self.plot_view = PlotView(self, self.core)
        self.plot_view.grid(row=0, column=1, sticky='NSWE')


    def _color_specimens(self, specimens):
        '''
        See _color_recording for reference.
        '''
        colors = []
        for specimen in specimens:
            analyser = self.core.get_manalyser(specimen, no_data_load=True)
            color = 'yellow'
            if analyser.are_rois_selected():
                color = 'green'
                if analyser.is_measured():
                    color = 'white'
            
            colors.append(color)
        return colors



    def copy_to_csv(self, formatted): 
        with open(os.path.join(GONIODIR, 'clipboard.csv'), 'w') as fp:
            fp.write(formatted.replace('\t', ','))


    def specimen_traces_to_clipboard(self, mean=False):
        '''
        If mean==True, copy only the average trace.
        Otherwise, copy all the traces of the fly.
        '''

        formatted = '' 
        
        # Always first clear clipboard; If something goes wrong, the user
        # doesn't want to keep pasting old data thinking it's new.
        self.root.clipboard_clear()

        if self.core.selected_recording is None:
            return None

        data = []

        self.root.clipboard_append(formatted)
        
        for pos_folder in self.core.analyser.list_imagefolders():
            all_movements = self.core.analyser.get_movements_from_folder(pos_folder)
            
            for eye, movements in all_movements.items():
                for repetition in range(len(movements)):
                    mag = np.sqrt(np.array(movements[repetition]['x'])**2 + np.array(movements[repetition]['y'])**2)
                    data.append(mag)

        if mean:
            data = [np.mean(data, axis=0)]
        
        for i_frame in range(len(data[0])):
            formatted += '\t'.join([str(data[i_repeat][i_frame]) for i_repeat in range(len(data)) ]) + '\n'
        
        self.root.clipboard_append(formatted)
        self.copy_to_csv(formatted)       


     


    def _color_recordings(self, recordings):
        '''
        Returns a list of colours, each corresponding to a recording
        in recordings, based on wheter the ROIs have been selected or
        movements measured for the recording.

        yellow      No ROIs, no movements
        green       ROIs, no movements
        white       ROIs and movements
        '''
        colors = []
        for recording in recordings:
            color = 'yellow'
            if self.core.analyser.folder_has_rois(recording):
                color = 'green'
                if self.core.analyser.folder_has_movements(recording):
                    color = 'white'
            
            colors.append(color)
        return colors


    def on_specimen_selection(self, specimen):
        '''
        When a selection happens in the specimens listbox.
        '''
        self.specimen_control_frame.config(text=specimen)

        self.core.set_current_specimen(specimen)
        

        # Recordings box
        recordings = self.core.analyser.list_imagefolders()
        self.recording_box.enable()
        self.recording_box.set_selections(recordings, colors=self._color_recordings(recordings))
         
        
        # Logick to set buttons inactive/active and their texts
        if self.core.analyser.are_rois_selected(): 
  
            self.button_rois.config(text='Reselect ROIs')
            self.button_rois.config(bg=self.default_button_bg)
            
            self.button_measure.config(state=tk.NORMAL)
            
            self.button_one_roi.config(state=tk.NORMAL)
            
            # Enable image_folder buttons
            for button in self.buttons_frame_3.get_buttons():
                button.config(state=tk.NORMAL)

            if self.core.analyser.is_measured():
                self.button_measure.config(text='Remeasure movement')
                self.button_measure.config(bg=self.default_button_bg)
            else:
                self.button_measure.config(text='Measure movement')
                self.button_measure.config(bg='green')
        else:
            #self.recording_box.disable()

            self.button_rois.config(text='Select ROIs')
            self.button_rois.config(bg='yellow')

            self.button_measure.config(state=tk.DISABLED)
            self.button_measure.config(text='Measure movement')
            self.button_measure.config(bg=self.default_button_bg)

            self.button_one_roi.config(state=tk.DISABLED)
            
            # Disable image_folder buttons
            for button in self.buttons_frame_3.get_buttons():
                button.config(state=tk.DISABLED)



        if self.core.analyser.are_rois_selected():
            self.core.analyser.load_ROIs()

        # Loading cached analyses and setting the recordings listbox

        if self.core.analyser.is_measured():
            self.core.analyser.load_analysed_movements()
            #self.recording_box.enable()
            
        
        N_rois = self.core.analyser.count_roi_selected_folders()
        N_image_folders = len(self.core.analyser.list_imagefolders())
        self.status_rois.config(text='ROIs selected {}/{}'.format(N_rois, N_image_folders))
        
        try:
            self.correction = self.core.analyser.get_antenna_level_correction()
        except:
            self.correction = False
        if self.correction is not False:
            self.status_antenna_level.config(text='Zero corrected, {:.2f} degrees'.format(self.correction))
        else:
            self.status_antenna_level.config(text='Zero corrected FALSE')

        self.status_active_analysis.config(text='Active analysis: {}'.format(self.core.analyser.active_analysis))

        
        # FIXME Instead of destroyign tickbox, make changes to tk_steroids
        # so that the selections can be reset
        self.tickbox_analyses.grid_forget()
        self.tickbox_analyses.destroy()
        self.tickbox_analyses = TickboxFrame(self.specimen_control_frame, self.core.analyser.list_analyses(),
                defaults=[self.core.analyser.active_analysis == an for an in self.core.analyser.list_analyses()],
                ncols=4, callback=lambda: self.plot_view.update_plot(None))
        self.tickbox_analyses.grid(row=5, column=0, sticky='W')

        self.button_rois.config(state=tk.NORMAL)
        

    def on_recording_selection(self, selected_recording):
        '''
        When a selection happens in the recordings listbox.
        
        selected_recording      Name of the recording. If 'current', keeps the current
        '''
        if selected_recording == 'current':
            selected_recording = self.core.selected_recording
        else:
            self.core.set_selected_recording(selected_recording)

        
        print(self.core.analyser.get_recording_time(selected_recording))
        
        angles = [list(angles_from_fn(selected_recording))]
        to_degrees(angles)
        horizontal, vertical = angles[0]
        self.status_horizontal.config(text='Horizontal angle {:.2f} degrees'.format(horizontal))
        self.status_vertical.config(text='Vertical angle {:.2f} degrees'.format(vertical))
        

        # Plotting only the view we have currently open
        self.plot_view.update_plot()


    def update_specimen(self, changed_specimens=False):
        '''
        Updates GUI colors, button states etc. to right values.
        
        Call this if there has been changes to specimens/image_folders by an
        external process or similar.
        '''        
        if changed_specimens:
            specimens = self.core.list_specimens()
            self.specimen_box.set_selections(specimens, self._color_specimens(specimens))

        if self.core.current_specimen is not None:
            self.on_specimen_selection(self.core.current_specimen)





def main():
    
    if (sys.platform in ['win32', 'cygwin', 'msys']):
        ctypes.windll.shcore.SetProcessDpiAwareness(1)

    root = tk.Tk()
    root.title('Gonio analysis - Tkinter GUI - {}'.format(__version__))
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)
    root.minsize(800,600)
    ExamineView(root).grid(sticky='NSWE')
    root.mainloop()


if __name__ == "__main__":
    main()




import os
import json

from gonioanalysis.directories import PROCESSING_TEMPDIR
from gonioanalysis.drosom.analysing import MAnalyser
from movemeter.stacks import stackread, stackwrite
from movemeter import Movemeter

class StartposAnalyser(MAnalyser):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._movements_skelefn = self._movements_skelefn.replace('movements_', 'startpos_')
        self.active_analysis = ''

        self._startpos_stack = os.path.join(PROCESSING_TEMPDIR, 'startpos_stacks', self.folder)


    def measure_movement(self, eye, *args, max_movement=30, **kwargs):
        '''Measures how the first frame repeat ROI moves across the repeats.
        '''



        self.movements = {}
        
        if not os.path.exists(self._crops_savefn):
            self.selectROIs() 
        self.load_ROIs()
        
        #angles = []
        image_folders = []
        stacks = []
        ROIs = []

        
        # Pull out the first frames out of each repeats
        # (needed especially if a stack saved)
        for image_folder in self.list_imagefolders():
            #images = self.list_images(image_folder, absolute_path=True)
            images = self.stacks[image_folder.removeprefix('pos')]

            print(f'Image folder {image_folder} has following images in it')

            start_frames = []

            for fns in images:
                print(fns[0])
                stack = stackread(fns[0])
                start_frames.append( next(stack) )
            
            os.makedirs(self._startpos_stack, exist_ok=True)
            write_fn = os.path.join(self._startpos_stack, f'{image_folder}.tiff')
            with stackwrite(write_fn) as fp:
                for frame in start_frames:
                    fp.write(frame)
            
            rois = self.get_rois(image_folder)
            for roi in rois:
                ROIs.append([roi])
                stacks.append([write_fn])
                image_folders.append(image_folder)

            print(f'Wrote {write_fn}')


        meter = Movemeter(upscale=10, absolute_results=True)
        meter.set_data(stacks, ROIs)
        

        for stack_i, image_folder in enumerate(image_folders):
            print('Analysing {} eye, done {}/{} for this eye'.format(eye.upper(), stack_i+1, len(stacks)))
            print("Calculating ROI's movement...")
            x, y = meter.measure_movement(stack_i, max_movement=max_movement)[0]
            print('Done.')
            
            angle = image_folder.removeprefix('pos')
            if angle not in self.movements:
                self.movements[angle] = []

            self.movements[angle].append( {'x': x, 'y': y} )


        # Save movements
        with open(self._movements_savefn.format(eye), 'w') as fp:
            json.dump(self.movements, fp)
        

    def is_measured(self):
        fns = [self._movements_savefn.format(eye) for eye in self.eyes]
        return all([os.path.exists(fn) for fn in fns])
